@@ -20,19 +20,26 @@
 */
 package org.apache.derby.client;
 
+import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Iterator;
+import java.util.concurrent.CopyOnWriteArrayList;
 import javax.sql.ConnectionEvent;
 import javax.sql.ConnectionEventListener;
+import javax.sql.PooledConnection;
+import javax.sql.StatementEvent;
+import javax.sql.StatementEventListener;
 import org.apache.derby.client.am.ClientMessageId;
+import org.apache.derby.client.am.ClientConnection;
+import org.apache.derby.client.am.LogWriter;
+import org.apache.derby.client.am.LogicalConnection;
 import org.apache.derby.client.am.SqlException;
 import org.apache.derby.client.am.stmtcache.JDBCStatementCache;
-import org.apache.derby.client.net.NetLogWriter;
 import org.apache.derby.client.net.NetXAConnection;
 import org.apache.derby.iapi.error.ExceptionSeverity;
-import org.apache.derby.jdbc.ClientBaseDataSourceRoot;
+import org.apache.derby.jdbc.BasicClientDataSource40;
 import org.apache.derby.jdbc.ClientDriver;
 import org.apache.derby.shared.common.reference.SQLState;
 
@@ -40,7 +47,7 @@ import org.apache.derby.shared.common.reference.SQLState;
  * A physical connection to a data source, to be used for creating logical
  * connections to the same data source.
  */
-public class ClientPooledConnection implements javax.sql.PooledConnection {
+public class ClientPooledConnection implements PooledConnection {
 
     /** Tells if this pooled connection is newly created. */
     private boolean newPC_ = true;
@@ -59,9 +66,8 @@ public class ClientPooledConnection implements javax.sql.PooledConnection {
      */
     private int eventIterators;
 
-    org.apache.derby.client.am.Connection physicalConnection_ = null;
-    org.apache.derby.client.net.NetConnection netPhysicalConnection_ = null;
-    org.apache.derby.client.net.NetXAConnection netXAPhysicalConnection_ = null;
+    ClientConnection physicalConnection_ = null;
+    NetXAConnection netXAPhysicalConnection_ = null;
 
     /**
      * The statement cache for the underlying physical connection.
@@ -72,13 +78,22 @@ public class ClientPooledConnection implements javax.sql.PooledConnection {
 
     /** The logical connection using the physical connection. */
     //@GuardedBy("this")
-    org.apache.derby.client.am.LogicalConnection logicalConnection_ = null;
+    private LogicalConnection logicalConnection_ = null;
 
-    protected org.apache.derby.client.am.LogWriter logWriter_ = null;
+    protected LogWriter logWriter_ = null;
 
-    /** Resource manager identificator. */
+    /** Resource manager identifier. */
     protected int rmId_ = 0;
 
+    /**
+     * List of statement event listeners. The list is copied on each write,
+     * ensuring that it can be safely iterated over even if other threads or
+     * the listeners fired in the same thread add or remove listeners.
+     */
+    private final CopyOnWriteArrayList<StatementEventListener>
+            statementEventListeners =
+                    new CopyOnWriteArrayList<StatementEventListener>();
+
     /**
      * Constructor for non-XA pooled connections.
      * <p>
@@ -92,8 +107,8 @@ public class ClientPooledConnection implements javax.sql.PooledConnection {
      * @throws SQLException if creating the pooled connection fails due problems
      *      in the database, or problems communicating with the database
      */
-    public ClientPooledConnection(ClientBaseDataSourceRoot ds,
-                                  org.apache.derby.client.am.LogWriter logWriter,
+    public ClientPooledConnection(BasicClientDataSource40 ds,
+                                  LogWriter logWriter,
                                   String user,
                                   String password) throws SQLException {
         logWriter_ = logWriter;
@@ -113,19 +128,18 @@ public class ClientPooledConnection implements javax.sql.PooledConnection {
             //PooledConnection which will then raise the events
             //on the listeners
             
-            netPhysicalConnection_ = (org.apache.derby.client.net.NetConnection)
+            physicalConnection_ =
             ClientDriver.getFactory().newNetConnection(
-                    (NetLogWriter) logWriter_,
+                    logWriter_,
                     user,
                     password,
                     ds,
                     -1,
                     false,
-                    this);
+                    ClientPooledConnection.this);
         } catch (SqlException se) {
             throw se.getSQLException();
         }
-        physicalConnection_ = netPhysicalConnection_;
     }
 
     /**
@@ -142,8 +156,8 @@ public class ClientPooledConnection implements javax.sql.PooledConnection {
      * @throws SQLException if creating the pooled connection fails due problems
      *      in the database, or problems communicating with the database
      */
-    public ClientPooledConnection(ClientBaseDataSourceRoot ds,
-                                  org.apache.derby.client.am.LogWriter logWriter,
+    public ClientPooledConnection(BasicClientDataSource40 ds,
+                                  LogWriter logWriter,
                                   String user,
                                   String password,
                                   int rmId) throws SQLException {
@@ -159,11 +173,14 @@ public class ClientPooledConnection implements javax.sql.PooledConnection {
         }
 
         try {
-            netXAPhysicalConnection_ = getNetXAConnection(ds,
-                    (NetLogWriter) logWriter_,
+            netXAPhysicalConnection_ = new NetXAConnection(
+                    logWriter,
                     user,
                     password,
-                    rmId);
+                    ds,
+                    rmId,
+                    true,
+                    this);
         } catch ( SqlException se ) {
             throw se.getSQLException();
         }
@@ -179,16 +196,23 @@ public class ClientPooledConnection implements javax.sql.PooledConnection {
         return this.statementCache != null;
     }
 
-    protected void finalize() throws java.lang.Throwable {
+    protected void finalize() throws Throwable {
         if (logWriter_ != null) {
             logWriter_.traceEntry(this, "finalize");
         }
-        close();
+
+        try {
+            close();
+        } finally {
+            // Any exception ignored if thrown from finalizer anyway, so no
+            // need to catch it.
+            super.finalize();
+        }
     }
 
     /**
      * Closes the physical connection to the data source and frees all
-     * assoicated resources.
+     * associated resources.
      * 
      * @throws SQLException if closing the connection causes an error. Note that
      *      this connection can still be considered closed even if an error
@@ -229,7 +253,7 @@ public class ClientPooledConnection implements javax.sql.PooledConnection {
      * 
      * @throws SQLException if creating a new logical connection fails
      */
-    public synchronized java.sql.Connection getConnection() throws SQLException {
+    public synchronized Connection getConnection() throws SQLException {
         try
         {
             if (logWriter_ != null) {
@@ -417,66 +441,75 @@ public class ClientPooledConnection implements javax.sql.PooledConnection {
     public synchronized void nullLogicalConnection() {
         logicalConnection_ = null;
     }
-    
-    /*-----------------------------------------------------------------*/
-    /*
-     * These methods are needed to provide StatementEvent support for 
-     * derby. 
-     * They are actually implemented in EmbedPooledConnection40 but have
-     * a dummy implementation here
-     */
-    
+
+    // JDBC 4.0 methods
+
     /**
+     * Registers a StatementEventListener with this PooledConnection object.
+     * Components that wish to be informed of events associated with the
+     * PreparedStatement object created by this PooledConnection like the close
+     * or error occurred event can register a StatementEventListener with this
+     * PooledConnection object.
      *
-     * The onStatementClose contains the logic for raising the Statement Closed
-     * events. This method has a dummy implementation here to avoid error when
-     * this class is compiled with jdk1.4. The class the actual implementation 
-     * in ClientPooledConnection40.
+     * @param listener A component that implements the StatementEventListener
+     * interface and wants to be notified of Statement closed or or Statement
+     * error occurred events
+     */
+    public void addStatementEventListener(StatementEventListener listener) {
+        if (logWriter_ != null) {
+            logWriter_.traceEntry(this, "addStatementEventListener", listener);
+        }
+        if (listener != null) {
+            statementEventListeners.add(listener);
+        }
+    }
+
+    /**
+     * Removes the specified previously registered listener object from the list
+     * of components that would be informed of events with a PreparedStatement
+     * object.
+     *
+     * @param listener The previously registered event listener that needs to be
+     * removed from the list of components
+     */
+    public void removeStatementEventListener(StatementEventListener listener) {
+        if (logWriter_ != null) {
+            logWriter_.traceEntry(
+                    this, "removeConnectionEventListener", listener);
+        }
+        statementEventListeners.remove(listener);
+    }
+
+    /**
+     * Raise the statementClosed event for all the listeners when the
+     * corresponding events occurs.
      *
      * @param statement The PreparedStatement that was closed
-     *
      */
     public void onStatementClose(PreparedStatement statement) {
-        
+        if (!statementEventListeners.isEmpty()) {
+            StatementEvent event = new StatementEvent(this, statement);
+            for (StatementEventListener l : statementEventListeners) {
+                l.statementClosed(event);
+            }
+        }
     }
-    
+
     /**
-     * The method contains the logic for raising the Statement error occurred
-     * events. This method has a dummy implementation here to avoid error when
-     * this class is compiled with jdk1.4. The class the actual implementation 
-     * in ClientPooledConnection40.
+     * Raise the statementErrorOccurred event for all the listeners when the
+     * corresponding events occurs.
      *
-     * @param statement The PreparedStatement that was closed
-     * @param sqle      The SQLException associated with the error that caused
-     *                  the invalidation of this PreparedStatement
+     * @param statement The PreparedStatement on which error occurred
+     * @param sqle The SQLException associated with the error that caused the
+     * invalidation of the PreparedStatements
      */
     public void onStatementErrorOccurred(PreparedStatement statement,
-                    SQLException sqle) {
-        
-    }
-    
-    /**
-     * creates and returns NetXAConnection. 
-     * Overwrite this method to create different version of NetXAConnection
-     * @param ds 
-     * @param logWriter 
-     * @param user 
-     * @param password 
-     * @param rmId 
-     * @return NetXAConnection
-     */
-    protected NetXAConnection getNetXAConnection (ClientBaseDataSourceRoot ds,
-                                  NetLogWriter logWriter,
-                                  String user,
-                                  String password,
-                                  int rmId) throws SqlException {
-          return new NetXAConnection(logWriter,
-                    user,
-                    password,
-                    ds,
-                    rmId,
-                    true,
-                    this);
-        
+            SQLException sqle) {
+        if (!statementEventListeners.isEmpty()) {
+            StatementEvent event = new StatementEvent(this, statement, sqle);
+            for (StatementEventListener l : statementEventListeners) {
+                l.statementErrorOccurred(event);
+            }
+        }
     }
 }
