@@ -21,29 +21,27 @@
 
 package org.apache.derby.iapi.sql.dictionary;
 
-import org.apache.derby.iapi.sql.depend.DependencyManager;
-
-import org.apache.derby.iapi.db.Database;
-import org.apache.derby.iapi.types.DataTypeDescriptor;
-import org.apache.derby.iapi.types.NumberDataValue;
-import org.apache.derby.iapi.types.DataValueFactory;
-import org.apache.derby.iapi.services.daemon.IndexStatisticsDaemon;
-import org.apache.derby.iapi.sql.compile.Visitable;
-import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
-import org.apache.derby.iapi.sql.execute.ExecutionFactory;
-import org.apache.derby.iapi.error.StandardException;
-import org.apache.derby.iapi.store.access.TransactionController;
-import org.apache.derby.iapi.types.RowLocation;
-
+import java.sql.Types;
+import java.util.Dictionary;
+import java.util.Hashtable;
+import java.util.List;
 import org.apache.derby.catalog.DependableFinder;
 import org.apache.derby.catalog.TypeDescriptor;
 import org.apache.derby.catalog.UUID;
+import org.apache.derby.iapi.db.Database;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.services.daemon.IndexStatisticsDaemon;
 import org.apache.derby.iapi.services.uuid.UUIDFactory;
-
-import java.sql.Types;
-import java.util.Dictionary;
-import java.util.List;
-import java.util.Hashtable;
+import org.apache.derby.iapi.sql.compile.Visitable;
+import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
+import org.apache.derby.iapi.sql.depend.DependencyManager;
+import org.apache.derby.iapi.sql.execute.ExecutionFactory;
+import org.apache.derby.iapi.store.access.TransactionController;
+import org.apache.derby.iapi.types.DataTypeDescriptor;
+import org.apache.derby.iapi.types.DataValueDescriptor;
+import org.apache.derby.iapi.types.DataValueFactory;
+import org.apache.derby.iapi.types.NumberDataValue;
+import org.apache.derby.iapi.types.RowLocation;
 
 /**
  * The DataDictionary interface is used with the data dictionary to get
@@ -126,6 +124,15 @@ public interface DataDictionary
 	/** Derby 10.10 System Catalog version */
 	public static final int DD_VERSION_DERBY_10_10		= 220;
 
+	/** Derby 10.11 System Catalog version */
+	public static final int DD_VERSION_DERBY_10_11		= 230;
+
+	/** Derby 10.12 System Catalog version */
+	public static final int DD_VERSION_DERBY_10_12		= 240;
+
+	/** Derby 10.13 System Catalog version */
+	public static final int DD_VERSION_DERBY_10_13		= 250;
+
 	// general info
 	public	static	final	String	DATABASE_ID = "derby.databaseID";
 
@@ -209,6 +216,7 @@ public interface DataDictionary
 	public static final int CHECK_CONSTRAINT = 4;
 	public static final int DROP_CONSTRAINT = 5;
 	public static final int FOREIGNKEY_CONSTRAINT = 6;
+    public static final int MODIFY_CONSTRAINT = 7;
 
 	/** Modes returned from startReading() */
 	public static final int COMPILE_ONLY_MODE = 0;
@@ -418,7 +426,7 @@ public interface DataDictionary
 	 *
 	 * @param props   The persistent properties used to configure password hashing.
 	 */
-    public  PasswordHasher  makePasswordHasher( Dictionary props )
+    public  PasswordHasher  makePasswordHasher( Dictionary<?,?> props )
         throws StandardException;
     
 	/**
@@ -833,13 +841,13 @@ public interface DataDictionary
 
 	/**
 	 * Load up the constraint descriptor list for this table
-	 * descriptor and return it.  If the descriptor list
-	 * is already loaded up, it is retuned without further
+     * descriptor (or all) and return it.  If the descriptor list
+     * is already loaded up, it is returned without further
 	 * ado.
 	 *
-	 * @param td			The table descriptor.
-	 *
-	 * @return The ConstraintDescriptorList for the table
+     * @param td The table descriptor.
+     * @return   The ConstraintDescriptorList for the table. If null, return
+     *           a list of all the constraint descriptors in all schemas.
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
@@ -1082,8 +1090,7 @@ public interface DataDictionary
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
-	public List getAllSPSDescriptors()
-		throws StandardException;
+    public List<SPSDescriptor> getAllSPSDescriptors() throws StandardException;
 
 	/**
 	 * Get all the parameter descriptors for an SPS.
@@ -1099,7 +1106,7 @@ public interface DataDictionary
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-    public DataTypeDescriptor[] getSPSParams(SPSDescriptor spsd, List defaults)
+    public DataTypeDescriptor[] getSPSParams(SPSDescriptor spsd, List<DataValueDescriptor> defaults)
             throws StandardException;
 
 	/**
@@ -1199,6 +1206,21 @@ public interface DataDictionary
 	public TriggerDescriptor getTriggerDescriptor(String name, SchemaDescriptor sd)
 				throws StandardException;
 
+	public int[] examineTriggerNodeAndCols(
+			Visitable actionStmt,
+			String oldReferencingName,
+			String newReferencingName,
+			String triggerDefinition,
+			int[] referencedCols,
+			int[] referencedColsInTriggerAction,
+			int actionOffset,
+			TableDescriptor triggerTableDescriptor,
+			int triggerEventMask,
+            boolean createTriggerTime,
+            List<int[]> replacements
+			) throws StandardException;
+	
+
 	/**
 	 * This method does the job of transforming the trigger action plan text
 	 * as shown below. 
@@ -1308,6 +1330,16 @@ public interface DataDictionary
 	 * 	false if here because an invalidated row level trigger with 
 	 *  REFERENCEd columns has been fired and hence trigger action
 	 *  sql associated with SPSDescriptor may be invalid too.
+     *
+     * @param replacements a list that will be populated with objects that
+     *  describe how {@code triggerDefinition} has been transformed into
+     *  the returned SQL text. Each element in the list will contain four
+     *  integers. The first two describe the begin and end offset of the
+     *  replaced text in the {@code triggerDefinition}. The last two describe
+     *  the begin and end offset of the replacement text in the returned
+     *  string. The begin offsets are inclusive, whereas the end offsets are
+     *  exclusive. The list can be {@code null} if the caller does not care
+     *  about this information.
 	 * 
 	 * @return Transformed trigger action sql
 	 * @throws StandardException
@@ -1322,7 +1354,9 @@ public interface DataDictionary
 			int actionOffset,
 			TableDescriptor triggerTableDescriptor,
 			int triggerEventMask,
-			boolean createTriggerTime)
+            boolean createTriggerTime,
+            List<int[]> replacements,
+           	int[] cols)
 	throws StandardException;
 	
 
@@ -1339,7 +1373,7 @@ public interface DataDictionary
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
-	public GenericDescriptorList getTriggerDescriptors(TableDescriptor td)
+    public TriggerDescriptorList getTriggerDescriptors(TableDescriptor td)
 		throws StandardException;
 
 	/**
@@ -1394,8 +1428,10 @@ public interface DataDictionary
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
-	public Hashtable hashAllConglomerateDescriptorsByNumber(TransactionController tc)
-		throws StandardException;
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+    public Hashtable<Long, ConglomerateDescriptor>
+        hashAllConglomerateDescriptorsByNumber(TransactionController tc)
+            throws StandardException;
 
 	/**
 	 * Get all of the TableDescriptors in the database and hash them by TableId
@@ -1412,7 +1448,8 @@ public interface DataDictionary
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
-	public Hashtable hashAllTableDescriptorsByTableId(TransactionController tc)
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+    public Hashtable<UUID, TableDescriptor> hashAllTableDescriptorsByTableId(TransactionController tc)
 		throws StandardException;
 
 	/**
@@ -1576,7 +1613,7 @@ public interface DataDictionary
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
-	public List getDependentsDescriptorList(String dependentID)
+    List<DependencyDescriptor> getDependentsDescriptorList(String dependentID)
 		throws StandardException;
 
 	/**
@@ -1590,7 +1627,7 @@ public interface DataDictionary
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
-	public List getProvidersDescriptorList(String providerID)
+    List<DependencyDescriptor> getProvidersDescriptorList(String providerID)
 		throws StandardException;
 
 	/**
@@ -1602,7 +1639,7 @@ public interface DataDictionary
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
-	public List getAllDependencyDescriptorsList()
+    public List<TupleDescriptor> getAllDependencyDescriptorsList()
 				throws StandardException;
 
 	/** 
@@ -1678,8 +1715,10 @@ public interface DataDictionary
 	/**
 		Get the list of routines matching the schema and routine name.
 	 */
-	public java.util.List getRoutineList(String schemaID, String routineName, char nameSpace)
-			throws StandardException;	
+    public List<AliasDescriptor> getRoutineList(
+        String schemaID,
+        String routineName,
+        char nameSpace) throws StandardException;
 	
 	/** 
 	 * Drop an AliasDescriptor from the DataDictionary
@@ -1772,7 +1811,7 @@ public interface DataDictionary
 	 * 
 	 * @return  array of row locations, null if table has no autoinc columns.
 	 *
-	 * @exception StandardException standard exception on error.
+     * @exception StandardException if an error happens
 	 */
 	public RowLocation[] computeAutoincRowLocations(TransactionController tc,
 											   TableDescriptor td)
@@ -1823,6 +1862,42 @@ public interface DataDictionary
 									  boolean incrementNeeded)
 		throws StandardException;
 	
+	/**
+	 * Computes the RowLocation in SYSSEQUENCES for a particular sequence. Also
+     * constructs the sequence descriptor.
+	 * 
+	 * @param tc			Transaction Controller to use.
+	 * @param sequenceIDstring UUID of the sequence as a string
+	 * @param rowLocation OUTPUT param for returing the row location
+	 * @param sequenceDescriptor OUTPUT param for return the sequence descriptor
+     *
+	 * @exception StandardException thrown on failure.
+	 */ 
+	public void computeSequenceRowLocation
+        ( TransactionController tc, String sequenceIDstring, RowLocation[] rowLocation, SequenceDescriptor[] sequenceDescriptor )
+		throws StandardException;
+    
+	/**
+	 * Set the current value of an ANSI/ISO sequence. This method does not perform
+     * any sanity checking but assumes that the caller knows what they are doing. If the
+     * old value on disk is not what we expect it to be, then we are in a race with another
+     * session. They won and we don't update the value on disk. However, if the old value
+     * is null, that is a signal to us that we should update the value on disk anyway.
+	 * 
+	 * @param tc			Transaction Controller to use.
+	 * @param rowLocation Row in SYSSEQUENCES to update.
+     * @param wait True if we should wait for locks
+     * @param oldValue What we expect to find in the CURRENTVALUE column.
+     * @param newValue What to stuff into the CURRENTVALUE column.
+	 * 
+	 * @return Returns true if the value was successfully updated, false if we lost a race with another session.
+     *
+	 * @exception StandardException thrown on failure.
+	 */
+    public  boolean updateCurrentSequenceValue
+        ( TransactionController tc, RowLocation rowLocation, boolean wait, Long oldValue, Long newValue )
+        throws StandardException;
+    
 	/**
 	 * Get the next number from an ANSI/ISO sequence generator
      * which was created with the CREATE SEQUENCE statement. May
@@ -1841,6 +1916,14 @@ public interface DataDictionary
         ( String sequenceUUIDstring, NumberDataValue returnValue )
         throws StandardException;
 
+    /**
+     * <p>
+     * Peek at the next value which will be returned by an identity generator.
+     * </p>
+     */
+    public Long peekAtIdentity( String schemaName, String tableName )
+        throws StandardException;
+
     /**
      * <p>
      * Peek at the next value which will be returned by a sequence generator.
@@ -1852,7 +1935,7 @@ public interface DataDictionary
 	/**
 	 * Gets all statistics Descriptors for a given table.
 	 */
-	public List getStatisticsDescriptors(TableDescriptor td)
+	public List<StatisticsDescriptor> getStatisticsDescriptors(TableDescriptor td)
 		throws StandardException;
 
 	/**
@@ -2361,4 +2444,26 @@ public interface DataDictionary
      */
     public DependableFinder getColumnDependableFinder(
             int formatId, byte[] columnBitMap);
+
+    /**
+     * Get the identity generator used to support the bulk-insert optimization
+     * in InsertResultSet.
+     *
+     * @param sequenceUUIDString UUID of the sequence which backs the identity column.
+     * @param restart   True if the counter should be re-initialized to its start position.
+     */
+    public  BulkInsertCounter   getBulkInsertCounter
+        ( String sequenceUUIDString, boolean restart )
+        throws StandardException;
+
+    /**
+     * Flush the updated values of the BulkInsertCounter to disk and to the original, cached
+     * SequenceUpdater. This is used for the bulk-insert optimization in InsertResultSet.
+     *
+     * @param sequenceUUIDString UUID of the sequence which backs the identity column.
+     * @param bic   the BulkInsertCounter which generates identities for bulk insert
+     */
+    public  void   flushBulkInsertCounter
+        ( String sequenceUUIDString, BulkInsertCounter bic )
+        throws StandardException;
 }
