@@ -26,7 +26,6 @@ import java.io.ObjectInput;
 import java.io.ObjectOutput;
 import java.sql.Types;
 import java.text.RuleBasedCollator;
-
 import org.apache.derby.catalog.TypeDescriptor;
 import org.apache.derby.catalog.types.RowMultiSetImpl;
 import org.apache.derby.catalog.types.TypeDescriptorImpl;
@@ -38,7 +37,7 @@ import org.apache.derby.iapi.services.io.Formatable;
 import org.apache.derby.iapi.services.io.StoredFormatIds;
 import org.apache.derby.iapi.services.loader.ClassFactory;
 import org.apache.derby.iapi.services.loader.ClassInspector;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.conn.ConnectionUtil;
 
 /** 
@@ -92,7 +91,16 @@ public final class DataTypeDescriptor implements Formatable
      */
     public static final DataTypeDescriptor SMALLINT_NOT_NULL =
         SMALLINT.getNullabilityType(false);
-     
+
+    /**
+     * Runtime INTEGER type that is nullable.
+     */
+    public static final DataTypeDescriptor DOUBLE =
+        new DataTypeDescriptor(TypeId.DOUBLE_ID, true);
+    
+    public  static  final   int MIN_VALUE_IDX = 0;
+    public  static  final   int MAX_VALUE_IDX = MIN_VALUE_IDX + 1;
+    public  static  final   int MAX_MIN_ARRAY_SIZE = MAX_VALUE_IDX + 1;
 
 	/*
 	** Static creators
@@ -1313,7 +1321,7 @@ public final class DataTypeDescriptor implements Formatable
 	 * Check if this type is comparable with the passed type.
 	 * 
 	 * @param compareWithDTD the type of the instance to compare with this type.
-	 * @param forEquals True if this is an = or <> comparison, false
+	 * @param forEquals True if this is an = or != comparison, false
 	 *					otherwise.
 	 * @param cf		A ClassFactory
 	 * @return true if compareWithDTD is comparable to this type, else false.
@@ -1491,6 +1499,8 @@ public final class DataTypeDescriptor implements Formatable
 		switch (typeId.getTypeFormatId())
 		{
 			case StoredFormatIds.LONGVARBIT_TYPE_ID:
+            //DERBY-6096 Need to estimate usage for BLOB and clob
+            case StoredFormatIds.BLOB_TYPE_ID:
 				/* Who knows?  Let's just use some big number */
 				return 10000.0;
 
@@ -1505,6 +1515,8 @@ public final class DataTypeDescriptor implements Formatable
 				return (double) (2.0 * getMaximumWidth());
 
 			case StoredFormatIds.LONGVARCHAR_TYPE_ID:
+            //DERBY-6096 Need to estimate usage for BLOB and clob
+            case StoredFormatIds.CLOB_TYPE_ID:
 				/* Who knows? Let's just use some big number */
 				return 10000.0;
 
@@ -1521,7 +1533,7 @@ public final class DataTypeDescriptor implements Formatable
 			case StoredFormatIds.INT_TYPE_ID:
 				return 4.0;
 
-			case StoredFormatIds.LONGINT_TYPE_ID:
+            case StoredFormatIds.BIGINT_TYPE_ID:
 				return 8.0;
 
 			case StoredFormatIds.REAL_TYPE_ID:
@@ -1886,5 +1898,38 @@ public final class DataTypeDescriptor implements Formatable
         }
         return name;    
     }
+
+    /**
+     * Get the maximum and minimum value for a fixed numeric type.
+     * Throws an unimplemented feature exception for a non-numeric type.
+     */
+    public  long[]  getNumericBounds()
+        throws StandardException
+    {
+        long[]  retval = new long[ MAX_MIN_ARRAY_SIZE ];
+
+        if ( getTypeId().equals( TypeId.SMALLINT_ID ) )
+        {
+            retval[ MIN_VALUE_IDX ] = Long.valueOf( Short.MIN_VALUE );
+            retval[ MAX_VALUE_IDX ] = Long.valueOf( Short.MAX_VALUE );
+        }
+        else if ( getTypeId().equals( TypeId.INTEGER_ID ) )
+        {
+            retval[ MIN_VALUE_IDX ] = Long.valueOf( Integer.MIN_VALUE );
+            retval[ MAX_VALUE_IDX ] = Long.valueOf( Integer.MAX_VALUE );
+        } else if ( getTypeId().equals( TypeId.BIGINT_ID ) )
+        {
+            // Could only be BIGINT
+            retval[ MIN_VALUE_IDX ] = Long.MIN_VALUE;
+            retval[ MAX_VALUE_IDX ] = Long.MAX_VALUE;
+        }
+        else
+        {
+            throw StandardException.newException( SQLState.BTREE_UNIMPLEMENTED_FEATURE );
+        }
+
+        return retval;
+    }
+    
 }
 
