@@ -25,7 +25,7 @@ package org.apache.derby.impl.jdbc;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.jdbc.EngineLOB;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.types.DataValueDescriptor;
 import org.apache.derby.iapi.types.RawToBinaryFormatStream;
 import org.apache.derby.iapi.types.Resetable;
@@ -318,7 +318,7 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
             // Nothing to do here, except checking if the position is valid.
             if (logicalPos >= control.getLength()) {
                 throw StandardException.newException(
-                        SQLState.BLOB_POSITION_TOO_LARGE, new Long(logicalPos));
+                        SQLState.BLOB_POSITION_TOO_LARGE, logicalPos);
             }
         } else {
             // Reposition the store stream, account for the length field offset.
@@ -328,7 +328,7 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
             } catch (EOFException eofe) {
                 throw StandardException.newException(
                         SQLState.BLOB_POSITION_TOO_LARGE, eofe,
-                        new Long(logicalPos));
+                        logicalPos);
             }
         }
         return logicalPos;
@@ -463,10 +463,10 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
         {
             if (startPos < 1)
                 throw StandardException.newException(
-                    SQLState.BLOB_BAD_POSITION, new Long(startPos));
+                    SQLState.BLOB_BAD_POSITION, startPos);
             if (length < 0)
                 throw StandardException.newException(
-                    SQLState.BLOB_NONPOSITIVE_LENGTH, new Integer(length));
+                    SQLState.BLOB_NONPOSITIVE_LENGTH, length);
 
             byte[] result;
             // if the blob is materialized
@@ -526,7 +526,7 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
         {  // if this is a setPosition exception then we ran out of Blob
             if (e.getMessageId().equals(SQLState.BLOB_LENGTH_TOO_LONG))
                 e = StandardException.newException(
-                    SQLState.BLOB_POSITION_TOO_LARGE, new Long(startPos));
+                    SQLState.BLOB_POSITION_TOO_LARGE, startPos);
             throw handleMyExceptions(e);
         }
         catch (Throwable t)
@@ -605,7 +605,7 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
    * Determines the byte position at which the specified byte
    * <code>pattern</code> begins within the <code>BLOB</code>
    * value that this <code>Blob</code> object represents.  The
-   * search for <code>pattern</code. begins at position
+   * search for <code>pattern</code>. begins at position
    * <code>start</code>
    * @param pattern the byte array for which to search
    * @param start the position at which to begin searching; the
@@ -627,7 +627,7 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
         {
             if (start < 1)
                 throw StandardException.newException(
-                    SQLState.BLOB_BAD_POSITION, new Long(start));
+                    SQLState.BLOB_BAD_POSITION, start);
             if (pattern == null)
                 throw StandardException.newException(SQLState.BLOB_NULL_PATTERN_OR_SEARCH_STR);
             if (pattern.length == 0)
@@ -729,7 +729,7 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
         {
             if (start < 1)
                 throw StandardException.newException(
-                    SQLState.BLOB_BAD_POSITION, new Long(start));
+                    SQLState.BLOB_BAD_POSITION, start);
             if (pattern == null)
                 throw StandardException.newException(SQLState.BLOB_NULL_PATTERN_OR_SEARCH_STR);
 
@@ -918,25 +918,25 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
 
         if (pos - 1 > length())
             throw Util.generateCsSQLException(SQLState.BLOB_POSITION_TOO_LARGE,
-                    new Long(pos));
+                    pos);
         if (pos < 1)
             throw Util.generateCsSQLException(SQLState.BLOB_BAD_POSITION,
-                    new Long(pos));
+                    pos);
         
         if ((offset < 0) || offset > bytes.length) {
             throw Util.generateCsSQLException(SQLState.BLOB_INVALID_OFFSET,
-                    new Long(offset));
+                    offset);
         }
         if (len < 0) {
             throw Util.generateCsSQLException(SQLState.BLOB_NONPOSITIVE_LENGTH,
-                    new Long(len));
+                    len);
         }
         if (len == 0) {
             return 0;
         }
         if (len > bytes.length - offset) {
             throw Util.generateCsSQLException(SQLState.BLOB_LENGTH_TOO_LONG,
-                    new Long(len));
+                    len);
         }
 
         try {
@@ -973,10 +973,10 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
             checkValidity ();
             if (pos - 1 > length())
                 throw Util.generateCsSQLException(
-                    SQLState.BLOB_POSITION_TOO_LARGE, new Long(pos));
+                    SQLState.BLOB_POSITION_TOO_LARGE, pos);
             if (pos < 1)
                 throw Util.generateCsSQLException(
-                    SQLState.BLOB_BAD_POSITION, new Long(pos));
+                    SQLState.BLOB_BAD_POSITION, pos);
             try {
                 if (materialized) {
                     return control.getOutputStream (pos - 1);
@@ -1015,7 +1015,7 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
 	{
             if (len > length())
                 throw Util.generateCsSQLException(
-                    SQLState.BLOB_LENGTH_TOO_LONG, new Long(len));
+                    SQLState.BLOB_LENGTH_TOO_LONG, len);
             try {
                 if (materialized) {
                     control.truncate (len);
@@ -1107,17 +1107,17 @@ final class EmbedBlob extends ConnectionChild implements Blob, EngineLOB
         if (pos <= 0) {
             throw Util.generateCsSQLException(
                     SQLState.BLOB_BAD_POSITION,
-                    new Long(pos));
+                    pos);
         }
         if (length < 0) {
             throw Util.generateCsSQLException(
                     SQLState.BLOB_NONPOSITIVE_LENGTH,
-                    new Long(length));
+                    length);
         }
         if (length > (this.length() - (pos -1))) {
             throw Util.generateCsSQLException(
                     SQLState.POS_AND_LENGTH_GREATER_THAN_LOB,
-                    new Long(pos), new Long(length));
+                    pos, length);
         }
         
         try {
