@@ -21,72 +21,67 @@
 
 package org.apache.derby.impl.services.monitor;
 
-import org.apache.derby.iapi.services.monitor.Monitor;
-import org.apache.derby.iapi.services.monitor.ModuleFactory;
-import org.apache.derby.iapi.services.monitor.ModuleControl;
-import org.apache.derby.iapi.services.monitor.ModuleSupportable;
-
-import org.apache.derby.iapi.services.monitor.PersistentService;
-
-import org.apache.derby.iapi.services.io.FormatIdUtil;
-import org.apache.derby.iapi.services.io.RegisteredFormatIds;
-import org.apache.derby.iapi.services.io.StoredFormatIds;
-
-import org.apache.derby.iapi.services.context.ContextManager;
-import org.apache.derby.iapi.services.context.Context;
-import org.apache.derby.iapi.services.context.ContextService;
-
-import org.apache.derby.iapi.services.stream.InfoStreams;
-import org.apache.derby.iapi.services.stream.PrintWriterGetHeader;
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import java.io.BufferedInputStream;
+import java.io.BufferedReader;
+import java.io.ByteArrayInputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.InputStreamReader;
+import java.io.PrintWriter;
+import java.lang.reflect.Constructor;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
+import java.net.URL;
+import java.security.PrivilegedAction;
+import java.security.AccessController;
+import java.security.AccessControlException;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Date;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Locale;
+import java.util.NoSuchElementException;
+import java.util.Properties;
+import java.util.ResourceBundle;
+import java.util.StringTokenizer;
+import java.util.Vector;
 import org.apache.derby.iapi.error.ErrorStringBuilder;
-import org.apache.derby.iapi.error.ShutdownException;
+import org.apache.derby.iapi.error.ExceptionSeverity;
+import org.apache.derby.shared.common.error.ShutdownException;
 import org.apache.derby.iapi.error.StandardException;
-import org.apache.derby.iapi.services.uuid.UUIDFactory;
-import org.apache.derby.iapi.services.timer.TimerFactory;
+import org.apache.derby.iapi.reference.Attribute;
 import org.apache.derby.iapi.reference.MessageId;
 import org.apache.derby.iapi.reference.Module;
 import org.apache.derby.iapi.reference.Property;
 import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.reference.Attribute;
-import org.apache.derby.iapi.services.property.PropertyUtil;
-
-import org.apache.derby.iapi.services.io.AccessibleByteArrayOutputStream;
-import org.apache.derby.iapi.services.loader.ClassInfo;
-import org.apache.derby.iapi.services.loader.InstanceGetter;
-import org.apache.derby.iapi.services.io.FormatableInstanceGetter;
-import org.apache.derby.iapi.error.ExceptionSeverity;
-
-import  org.apache.derby.io.StorageFactory;
-
-import org.apache.derby.iapi.services.info.JVMInfo;
+import org.apache.derby.iapi.services.context.Context;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.context.ContextService;
 import org.apache.derby.iapi.services.i18n.BundleFinder;
 import org.apache.derby.iapi.services.i18n.MessageService;
-
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.BufferedInputStream;
-import java.io.PrintWriter;
-import java.io.BufferedReader;
-import java.io.InputStreamReader;
-import java.io.ByteArrayInputStream;
-
-import java.util.Collections;
-import java.util.Date;
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.Properties;
-import java.util.Enumeration;
-import java.util.StringTokenizer;
-import java.util.Vector;
-import java.util.Locale;
-import java.util.ResourceBundle;
-import java.util.NoSuchElementException;
-
-import java.lang.reflect.InvocationTargetException;
-
-import java.net.URL;
+import org.apache.derby.iapi.services.info.JVMInfo;
+import org.apache.derby.iapi.services.io.AccessibleByteArrayOutputStream;
+import org.apache.derby.iapi.services.io.FormatIdUtil;
+import org.apache.derby.iapi.services.io.FormatableInstanceGetter;
+import org.apache.derby.iapi.services.io.RegisteredFormatIds;
+import org.apache.derby.iapi.services.io.StoredFormatIds;
+import org.apache.derby.iapi.services.loader.ClassInfo;
+import org.apache.derby.iapi.services.loader.InstanceGetter;
+import org.apache.derby.iapi.services.monitor.ModuleControl;
+import org.apache.derby.iapi.services.monitor.ModuleFactory;
+import org.apache.derby.iapi.services.monitor.ModuleSupportable;
+import org.apache.derby.iapi.services.monitor.Monitor;
+import org.apache.derby.iapi.services.monitor.PersistentService;
+import org.apache.derby.iapi.services.property.PropertyUtil;
+import org.apache.derby.iapi.services.stream.InfoStreams;
+import org.apache.derby.iapi.services.stream.PrintWriterGetHeader;
+import org.apache.derby.iapi.services.timer.TimerFactory;
+import org.apache.derby.iapi.services.uuid.UUIDFactory;
+import org.apache.derby.io.StorageFactory;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 /**
 	Implementation of the monitor that uses the class loader
@@ -102,16 +97,17 @@ abstract class BaseMonitor
 	/**
 		Hash table of objects that implement PersistentService keyed by their getType() method.
 	*/
-	private HashMap serviceProviders = new HashMap();
+    private final HashMap<String,PersistentService> serviceProviders =
+            new HashMap<String,PersistentService>();
 	private static final String LINE = 
         "----------------------------------------------------------------";
 
 	// Vector of class objects of implementations, found in the System, application
 	// and default (modules.properties) properties
 
-	Vector[]     implementationSets;
+    private List<List<Class<?>>> implementationSets;
 
-	private Vector	  services;					// Vector of TopServices
+    private final Vector<TopService> services; // Vector of TopServices
 
 	Properties bootProperties;		// specifc properties provided by the boot method, override everything else
 	Properties applicationProperties;
@@ -140,7 +136,7 @@ abstract class BaseMonitor
 	BaseMonitor() {
 		super();
 
-		services = new Vector(0, 1);
+		services = new Vector<TopService>(0, 1);
 		services.add(new TopService(this));	// first element is always the free-floating service
 	}
 
@@ -182,7 +178,7 @@ abstract class BaseMonitor
 				if (position == 0)
 					break;
 
-				ts = (TopService) services.get(position);
+				ts = services.get(position);
 			}
 
 			// push a new context manager
@@ -203,10 +199,9 @@ abstract class BaseMonitor
 		}
 		
 		Monitor.getStream().println(LINE);
-		((TopService) services.get(0)).shutdown();
+		(services.get(0)).shutdown();
 
-		ContextService.stop();
-		Monitor.clearMonitor();
+		stopContextService();
 	}
 
 	/**
@@ -243,13 +238,14 @@ abstract class BaseMonitor
 		logging = log;
 
 		// false indicates the full monitor is required, not the lite.
-		if (!initialize(false))
-			return;
+        if (!initialize(false)) {
+            dumpTempWriter(true);
+            return;
+        }
 
 		// if monitor is already set then the system is already
 		// booted or in the process of booting or shutting down.
-		if (!Monitor.setMonitor(this))
-			return;
+		if ( setMonitor( this ) ) { return; }
 
 		MessageService.setFinder(this);
 
@@ -279,10 +275,10 @@ abstract class BaseMonitor
 			}
 		}
 
-		Vector bootImplementations = getImplementations(bootProperties, false);
+		Vector<Class<?>> bootImplementations = getImplementations(bootProperties, false);
 
-		Vector systemImplementations = null;
-		Vector applicationImplementations = null;
+		Vector<Class<?>> systemImplementations = null;
+		Vector<Class<?>> applicationImplementations = null;
 
 		// TEMP - making this sanity only breaks the unit test code
 		// I will fix soon, djd.
@@ -292,7 +288,7 @@ abstract class BaseMonitor
 			applicationImplementations = getImplementations(applicationProperties, false);
 		}
 
-		Vector defaultImplementations = getDefaultImplementations();
+        Vector<Class<?>> defaultImplementations = getDefaultImplementations();
 
 		int implementationCount = 0;
 		if (bootImplementations != null)
@@ -309,23 +305,22 @@ abstract class BaseMonitor
 
 		if (defaultImplementations != null)
 			implementationCount++;
-		implementationSets = new Vector[implementationCount];
 
-		implementationCount = 0;
+        implementationSets = new ArrayList<List<Class<?>>>(implementationCount);
+
 		if (bootImplementations != null)
-			implementationSets[implementationCount++] = bootImplementations;
+            implementationSets.add(bootImplementations);
 		
 		if (true || SanityManager.DEBUG) {
 			// Don't allow external code to override our implementations.
 			if (systemImplementations != null)
-				implementationSets[implementationCount++] = systemImplementations;
+                implementationSets.add(systemImplementations);
 			if (applicationImplementations != null)
-				implementationSets[implementationCount++] = applicationImplementations;
+                implementationSets.add(applicationImplementations);
 		}
 
 		if (defaultImplementations != null)
-			implementationSets[implementationCount++] = defaultImplementations;
-
+            implementationSets.add(defaultImplementations);
 
 		if (SanityManager.DEBUG) {
 			// Look for the derby.debug.* properties.
@@ -361,7 +356,10 @@ abstract class BaseMonitor
 			dumpTempWriter(true);
 
 			return;
-		}
+        } catch (AccessControlException e) {
+            dumpTempWriter(true);
+            throw e;
+        }
 
 		// switch cover to the real error stream and
 		// dump any messages we have been saving ...
@@ -417,7 +415,7 @@ abstract class BaseMonitor
 		TopService myts = null;
 		synchronized (this) {
 			for (int i = 1; i < services.size(); i++) {
-				TopService ts = (TopService) services.get(i);
+				TopService ts = services.get(i);
 				if (ts.isPotentialService(key)) {
 					myts = ts;
 					break;
@@ -548,10 +546,10 @@ abstract class BaseMonitor
 	private synchronized TopService findTopService(Object serviceModule) {
 
 		if (serviceModule == null)
-			return (TopService) services.get(0);
+			return services.get(0);
 
 		for (int i = 1; i < services.size(); i++) {
-			TopService ts = (TopService) services.get(i);
+			TopService ts = services.get(i);
 			if (ts.inService(serviceModule))
 				return ts;
 		}
@@ -602,14 +600,14 @@ abstract class BaseMonitor
 			off = fmtId - StoredFormatIds.MIN_TWO_BYTE_FORMAT_ID;
 			iga = rc2;
 			if (iga == null) {
-				iga = rc2 = new InstanceGetter[RegisteredFormatIds.TwoByte.length];
+				iga = rc2 = new InstanceGetter[ RegisteredFormatIds.countTwoByteIDs() ];
 			}
 
 			ig = iga[off];
 			if (ig != null) {
 				return ig;
 			}
-			className = RegisteredFormatIds.TwoByte[off];
+			className = RegisteredFormatIds.classNameForTwoByteID( off );
 
 		} catch (ArrayIndexOutOfBoundsException aioobe) {
 			className = null;
@@ -621,11 +619,12 @@ abstract class BaseMonitor
 
 			Throwable t;
 			try {
-				Class clazz = Class.forName(className);
+				Class<?> clazz = Class.forName(className);
+                final Constructor<?> constructor = clazz.getDeclaredConstructor();
 
 				// See if it is a FormatableInstanceGetter
 				if (FormatableInstanceGetter.class.isAssignableFrom(clazz)) {
-					FormatableInstanceGetter tfig = (FormatableInstanceGetter) clazz.newInstance();
+                    FormatableInstanceGetter tfig = (FormatableInstanceGetter) constructor.newInstance();
 					tfig.setFormatId(fmtId);
 					return iga[off] = tfig;
 				}
@@ -638,6 +637,10 @@ abstract class BaseMonitor
 				t = iae;
 			} catch (InstantiationException ie) {
 				t = ie;
+			} catch (NoSuchMethodException nsme) {
+				t = nsme;
+			} catch (InvocationTargetException ite) {
+				t = ite;
 			} catch (LinkageError le) {
 				t = le;
 			}
@@ -688,18 +691,19 @@ abstract class BaseMonitor
  		catch (IllegalAccessException iae) {
 			t = iae;
 		}
+ 		catch (NoSuchMethodException iae) {
+			t = iae;
+		}
 		catch (InvocationTargetException ite) {
 			t = ite;
-		}
+        }
 		catch (LinkageError le) {
 			t = le;
 		}
 		throw StandardException.newException(SQLState.REGISTERED_CLASS_INSTANCE_ERROR,
-			t, new Integer(identifier), "XX" /*ci.getClassName()*/);
+			t, identifier, "XX" /*ci.getClassName()*/);
 	}
 
-	private Boolean exceptionTrace;
-
 	/**
 		load a module instance.
 
@@ -708,18 +712,17 @@ abstract class BaseMonitor
 
 		The module's start or create method is not called.
 	*/
-
-	protected Object loadInstance(Class factoryInterface, Properties properties) {
+	protected Object loadInstance(Class<?> factoryInterface, Properties properties) {
 
 		Object instance = null;
 
-		Vector localImplementations = getImplementations(properties, false);
+		Vector<Class<?>> localImplementations = getImplementations(properties, false);
 		if (localImplementations != null) {
 			instance = loadInstance(localImplementations, factoryInterface, properties);
 		}
 
-		for (int i = 0; i < implementationSets.length; i++) {
-			instance = loadInstance(implementationSets[i], factoryInterface, properties);
+        for (List<Class<?>> set : implementationSets) {
+            instance = loadInstance(set, factoryInterface, properties);
 			if (instance != null)
 				break;
 		}
@@ -727,8 +730,9 @@ abstract class BaseMonitor
 		return instance;
 	}
 
-
-	private Object loadInstance(Vector implementations, Class factoryInterface, Properties properties) {
+    private Object loadInstance(List<Class<?>> implementations,
+                                Class<?> factoryInterface,
+                                Properties properties) {
 
 		for (int index = 0; true; index++) {
 
@@ -748,15 +752,16 @@ abstract class BaseMonitor
 
 	/**
 		Find a class that implements the required index, return the index
-		into the implementations vecotr of that class. Returns -1 if no class
+        into the implementations vector of that class. Returns -1 if no class
 		could be found.
 	*/
-	private static int findImplementation(Vector implementations, int startIndex, Class factoryInterface) {
+    private static int findImplementation(List<Class<?>> implementations,
+            int startIndex, Class<?> factoryInterface) {
 
 		for (int i = startIndex; i < implementations.size(); i++) {
 
 			//try {
-				Class factoryClass = (Class) implementations.get(i);
+				Class<?> factoryClass = implementations.get(i);
 				if (!factoryInterface.isAssignableFrom(factoryClass)) {
 					continue;
 				}
@@ -773,36 +778,36 @@ abstract class BaseMonitor
 	}
 
 	/**
-	*/
-	private Object newInstance(String className) {
+     * Return a new instance of class {@code classObject} using
+     * a no-param constructor.
+     * @param classObject the class to instantiate
+     * @return the instantiated object
+     */
+    private Object newInstance(Class<?> classObject) {
 
 		try {
+            final Constructor<?> constructor = classObject.getDeclaredConstructor();
+            final Object module = constructor.newInstance();
 
-			Class factoryClass = Class.forName(className);
-			return factoryClass.newInstance();
-		}
-		catch (ClassNotFoundException e) {
-			report(className + " " + e.toString());
-		}
-		catch (InstantiationException e) {
-			report(className + " " + e.toString());
-		}
- 		catch (IllegalAccessException e) {
-			report(className + " " + e.toString());
-		}
-		catch (LinkageError le) {
-			report(className + " " + le.toString());
-			reportException(le);
-		}
+            // Get and report any warnings generated during initialization
+            try {
+                final Method getWarn = classObject.getMethod("getWarnings");
+                final String warnings = (String)getWarn.invoke(module);
 
-		return null;
-	}
-	/**
-	*/
-	private Object newInstance(Class classObject) {
+                if (warnings != null) {
+                    report(warnings);
+                }
+            } catch (NoSuchMethodException e) {
+                // Ok, not all modules support this method
+            } catch (InvocationTargetException e) {
+                // Should never happen
+                if (SanityManager.DEBUG) {
+                    SanityManager.NOTREACHED();
+                }
+                report(e.toString());
+            }
 
-		try {
-			return classObject.newInstance();
+            return module;
 		}
 		catch (InstantiationException e) {
 			report(classObject.getName() + " " + e.toString());
@@ -810,6 +815,12 @@ abstract class BaseMonitor
  		catch (IllegalAccessException e) {
 			report(classObject.getName() + " " + e.toString());
 		}
+ 		catch (NoSuchMethodException e) {
+			report(classObject.getName() + " " + e.toString());
+		}
+ 		catch (InvocationTargetException e) {
+            report(classObject.getName() + " " + e.getCause().toString());
+		}
 		catch (LinkageError le) {
 			report(classObject.getName() + " " + le.toString());
 			reportException(le);
@@ -840,7 +851,7 @@ abstract class BaseMonitor
 
 			// count the number of services that implement the required protocol
 			for (int i = 1; i < services.size(); i++) {
-				ts = (TopService) services.get(i);
+				ts = services.get(i);
 				if (ts.isActiveService()) {
 					if (ts.getKey().getFactoryInterface().getName().equals(protocol))
 						count++;
@@ -852,7 +863,7 @@ abstract class BaseMonitor
 			if (count != 0) {
 				int j = 0;
 				for (int i = 1; i < services.size(); i++) {
-					ts = (TopService) services.get(i);
+					ts = services.get(i);
 					if (ts.isActiveService()) {
 						if (ts.getKey().getFactoryInterface().getName().equals(protocol)) {
 							list[j++] = ts.getServiceType().getUserServiceName(ts.getKey().getIdentifier());
@@ -1047,12 +1058,12 @@ abstract class BaseMonitor
 		If no implementations are listed in the properties object
 		then null is returned.
 	*/
-	private Vector getImplementations(Properties moduleList, boolean actualModuleList) {
+	private Vector<Class<?>> getImplementations(Properties moduleList, boolean actualModuleList) {
 
 		if (moduleList == null)
 			return null;
 
-		Vector implementations = actualModuleList ? new Vector(moduleList.size()) : new Vector(0,1);
+		Vector<Class<?>> implementations = actualModuleList ? new Vector<Class<?>>(moduleList.size()) : new Vector<Class<?>>(0,1);
 
 		// Get my current JDK environment
 		int theJDKId = JVMInfo.JDK_ID;
@@ -1126,7 +1137,7 @@ nextModule:
 			}
 
 			try {
-				Class possibleModule = Class.forName(className);
+				Class<?> possibleModule = Class.forName(className);
 
 				// Look for the monitors special modules, PersistentService ones.
 				if (getPersistentServiceImplementation(possibleModule))
@@ -1218,7 +1229,7 @@ nextModule:
 		return implementations;
 	}
 
-    private boolean getPersistentServiceImplementation( Class possibleModule)
+    private boolean getPersistentServiceImplementation( Class<?> possibleModule)
     {
         if( ! PersistentService.class.isAssignableFrom(possibleModule))
             return false;
@@ -1232,7 +1243,7 @@ nextModule:
         return true;
     } // end of getPersistentServiceImplementation
         
-	private Vector getDefaultImplementations() {
+	private Vector<Class<?>> getDefaultImplementations() {
 
 		Properties moduleList = getDefaultModuleProperties();
 
@@ -1408,9 +1419,9 @@ nextModule:
 	*/
 	private void determineSupportedServiceProviders() {
 
-		for (Iterator i = serviceProviders.values().iterator(); i.hasNext(); ) {
+		for (Iterator<PersistentService> i = serviceProviders.values().iterator(); i.hasNext(); ) {
 
-			Object provider = i.next();
+			PersistentService provider = i.next();
 
 			// see if this provider can live in this environment
 			if (!BaseMonitor.canSupport(provider, (Properties) null)) {
@@ -1651,19 +1662,13 @@ nextModule:
         String className = PropertyUtil.getSystemProperty( propertyName);
         if( className != null)
             return className;
-        return (String) storageFactories.get( subSubProtocol);
+        return storageFactories.get( subSubProtocol);
     } // end of getStorageFactoryClassName
 
-    private static final HashMap storageFactories = new HashMap();
+    private static final HashMap<String,String> storageFactories = new HashMap<String,String>();
     static {
-		String dirStorageFactoryClass;
-        if (!JVMInfo.J2ME)
-            dirStorageFactoryClass = "org.apache.derby.impl.io.DirStorageFactory4";
-        else
-            dirStorageFactoryClass = "org.apache.derby.impl.io.DirStorageFactory";
-
-
-        storageFactories.put( PersistentService.DIRECTORY, dirStorageFactoryClass);
+        storageFactories.put( PersistentService.DIRECTORY,
+                                "org.apache.derby.impl.io.DirStorageFactory");
         storageFactories.put( PersistentService.CLASSPATH,
                                 "org.apache.derby.impl.io.CPStorageFactory");
         storageFactories.put( PersistentService.JAR,
@@ -1724,7 +1729,7 @@ nextModule:
 				}
 
 				for (int i = 1; i < services.size(); i++) {
-					TopService ts2 = (TopService) services.get(i);
+					TopService ts2 = services.get(i);
 					if (ts2.isPotentialService(serviceKey)) {
 						// if the service already exists then  just return null
 						return null;
@@ -2064,7 +2069,7 @@ nextModule:
 	public ResourceBundle getBundle(String messageId) {
 		ContextManager cm;
 		try {
-			cm = ContextService.getFactory().getCurrentContextManager();
+			cm = getContextService().getCurrentContextManager();
 		} catch (ShutdownException se) {
 			cm = null;
 		}
@@ -2086,24 +2091,77 @@ nextModule:
 
 	}
 
-	public void setThreadPriority(int priority) {
+    @Override
+    public final boolean isDaemonThread(Thread thread) {
+        return thread.getThreadGroup() == daemonGroup;
+    }
 
-		Thread t = Thread.currentThread();
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextService    getContextService()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ContextService>()
+             {
+                 public ContextService run()
+                 {
+                     return ContextService.getFactory();
+                 }
+             }
+             );
+    }    
 
-		if (t.getThreadGroup() == daemonGroup) {
-			t.setPriority(priority);
-		}
-	}
+    /**
+     * Privileged shutdown of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  void    stopContextService()
+    {
+        AccessController.doPrivileged
+            (
+             new PrivilegedAction<Object>()
+             {
+                 public Object run()
+                 {
+                     ContextService.stop();
+                     Monitor.clearMonitor();
+                     return null;
+                 }
+             }
+             );
+    }    
 
-	/**
+    /**
+     * Privileged startup. Must be private so that user code
+     * can't call this entry point. Returns true if the system is
+     * already booted or in the process of shutting down.
+     */
+    private  static  boolean    setMonitor( final BaseMonitor baseMonitor )
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<Boolean>()
+             {
+                 public Boolean run()
+                 {
+                     return !Monitor.setMonitor( baseMonitor );
+                 }
+             }
+             ).booleanValue();
+    }
+
+    /**
 		Initialize the monitor wrt the current environemnt.
 		Returns false if the monitor cannot be initialized, true otherwise.
 	*/
 	abstract boolean initialize(boolean lite);
 
-    class ProviderEnumeration implements Enumeration
+    class ProviderEnumeration implements Enumeration<PersistentService>
     {
-        private Enumeration serviceProvidersKeys = (serviceProviders == null) ? null :
+        private Enumeration<String> serviceProvidersKeys = (serviceProviders == null) ? null :
             Collections.enumeration(serviceProviders.keySet());
         private Properties startParams;
         private Enumeration paramEnumeration;
@@ -2117,12 +2175,12 @@ nextModule:
                 paramEnumeration = startParams.keys();
         }
 
-        public Object nextElement() throws NoSuchElementException
+        public PersistentService nextElement() throws NoSuchElementException
         {
             if( serviceProvidersKeys != null && serviceProvidersKeys.hasMoreElements())
                 return serviceProviders.get( serviceProvidersKeys.nextElement());
             getNextStorageFactory();
-            Object ret = storageFactoryPersistentService;
+            PersistentService ret = storageFactoryPersistentService;
             storageFactoryPersistentService = null;
             return ret;
         }
