@@ -22,45 +22,27 @@
 package org.apache.derby.impl.sql.catalog;
 
 import org.apache.derby.iapi.services.monitor.Monitor;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.io.Formatable;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.sql.dictionary.CatalogRowFactory;
 import org.apache.derby.iapi.sql.dictionary.ConglomerateDescriptor;
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
-import org.apache.derby.iapi.sql.dictionary.SPSDescriptor;
 import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
 import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
-import org.apache.derby.iapi.sql.dictionary.AliasDescriptor;
-import org.apache.derby.iapi.types.DataValueFactory;
-import org.apache.derby.iapi.types.RowLocation;
 import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.store.access.ConglomerateController;
 import org.apache.derby.iapi.store.access.TransactionController;
-import org.apache.derby.iapi.sql.dictionary.IndexRowGenerator;
-import org.apache.derby.iapi.store.access.ScanController;
-import org.apache.derby.iapi.sql.execute.ExecRow;
-import org.apache.derby.iapi.sql.execute.ExecIndexRow;
 import org.apache.derby.iapi.services.io.StoredFormatIds;
 
-import org.apache.derby.iapi.services.io.FormatableBitSet;
-import org.apache.derby.iapi.services.info.ProductGenusNames;
 import org.apache.derby.iapi.services.info.ProductVersionHolder;
-import org.apache.derby.iapi.reference.JDBC30Translation;
-import org.apache.derby.iapi.reference.Limits;
 import org.apache.derby.iapi.util.IdUtil;
 
-import org.apache.derby.iapi.services.uuid.UUIDFactory;
-import org.apache.derby.catalog.UUID;
-import org.apache.derby.catalog.types.RoutineAliasInfo;
-import org.apache.derby.catalog.AliasInfo;
-import org.apache.derby.catalog.TypeDescriptor;
-import org.apache.derby.iapi.types.DataTypeDescriptor;
 import java.io.IOException;
 import java.io.ObjectInput;
 import java.io.ObjectOutput;
-import java.sql.Types;
-import java.util.Enumeration;
+import java.security.AccessController;
+import java.security.PrivilegedActionException;
+import java.security.PrivilegedExceptionAction;
 import java.util.HashSet;
 import java.util.Properties;
 
@@ -164,6 +146,12 @@ public	class DD_Version implements	Formatable
 			return "10.9";
 		case DataDictionary.DD_VERSION_DERBY_10_10:
 			return "10.10";
+		case DataDictionary.DD_VERSION_DERBY_10_11:
+			return "10.11";
+		case DataDictionary.DD_VERSION_DERBY_10_12:
+			return "10.12";
+		case DataDictionary.DD_VERSION_DERBY_10_13:
+			return "10.13";
 		default:
 			return null;
 		}
@@ -209,7 +197,7 @@ public	class DD_Version implements	Formatable
 
 		} else {
            
-			if (Monitor.isFullUpgrade(startParams, dictionaryVersion.toString())) {
+			if (isFullUpgrade(startParams, dictionaryVersion.toString())) {
 				performMajorUpgrade = true;
 			} else {
 				softUpgradeRun = true;
@@ -382,7 +370,7 @@ public	class DD_Version implements	Formatable
 		
 		*/
 
-        HashSet  newlyCreatedRoutines = new HashSet();
+        HashSet<String>  newlyCreatedRoutines = new HashSet<String>();
         
 		if (fromMajorVersionNumber <= DataDictionary.DD_VERSION_DERBY_10_3)
 		{
@@ -418,7 +406,7 @@ public	class DD_Version implements	Formatable
 
         if (fromMajorVersionNumber <= DataDictionary.DD_VERSION_DERBY_10_5)
         {
-            // On ugrade from versions before 10.6, create system procedures
+            // On upgrade from versions before 10.6, create system procedures
             // added in 10.6.
             bootingDictionary.create_10_6_system_procedures(tc,
                     newlyCreatedRoutines);
@@ -433,7 +421,7 @@ public	class DD_Version implements	Formatable
 
         if (fromMajorVersionNumber <= DataDictionary.DD_VERSION_DERBY_10_1)
         {
-            // On ugrade from versions before 10.2, create system procedures
+            // On upgrade from versions before 10.2, create system procedures
             // added in 10.2.
             bootingDictionary.create_10_2_system_procedures(
                 tc,
@@ -461,7 +449,7 @@ public	class DD_Version implements	Formatable
 
         if (fromMajorVersionNumber <= DataDictionary.DD_VERSION_DERBY_10_2)
         {
-            // On ugrade from versions before 10.3, create system procedures
+            // On upgrade from versions before 10.3, create system procedures
             // added in 10.3.
             bootingDictionary.create_10_3_system_procedures(tc, newlyCreatedRoutines );
         }
@@ -504,7 +492,7 @@ public	class DD_Version implements	Formatable
         
         if (fromMajorVersionNumber <= DataDictionary.DD_VERSION_DERBY_10_8)
         {
-            // On ugrade from versions before 10.9, create system procedures
+            // On upgrade from versions before 10.9, create system procedures
             // added in 10.9.
             bootingDictionary.create_10_9_system_procedures( tc, newlyCreatedRoutines );
             
@@ -522,11 +510,42 @@ public	class DD_Version implements	Formatable
 
         if (fromMajorVersionNumber <= DataDictionary.DD_VERSION_DERBY_10_9)
         {
-            // On ugrade from versions before 10.10, create system procedures
+            // On upgrade from versions before 10.10, create system procedures
             // added in 10.10.
             bootingDictionary.create_10_10_system_procedures( tc, newlyCreatedRoutines );
         }
 
+        if (fromMajorVersionNumber <= DataDictionary.DD_VERSION_DERBY_10_10)
+        {
+            // On upgrade from versions before 10.11, add a column to the
+            // SYSTRIGGERS table in order to support the WHEN clause.
+            bootingDictionary.upgrade_addColumns(
+              bootingDictionary.getNonCoreTIByNumber(
+                DataDictionary.SYSTRIGGERS_CATALOG_NUM).getCatalogRowFactory(),
+                new int[] { 18 }, tc);
+            
+            // On upgrade from versions before 10.11, create system procedures
+            // added in 10.11.
+            bootingDictionary.create_10_11_system_procedures( tc, newlyCreatedRoutines );
+
+            // Add a sequence generator for every identity column
+            bootingDictionary.createIdentitySequences( tc );
+        }
+
+        if (fromMajorVersionNumber <= DataDictionary.DD_VERSION_DERBY_10_11)
+        {
+            // On upgrade from versions before 10.12, create system procedures
+            // added in 10.12.
+            bootingDictionary.create_10_12_system_procedures( tc, newlyCreatedRoutines );
+        }
+
+	if (fromMajorVersionNumber <= DataDictionary.DD_VERSION_DERBY_10_12)
+        {
+            // On upgrade from versions before 10.13, create system procedures
+            // added in 10.13.
+	    bootingDictionary.create_10_13_system_procedures( tc, newlyCreatedRoutines );
+        }
+
         // Grant PUBLIC access to some system routines
         bootingDictionary.grantPublicAccessToSystemRoutines(newlyCreatedRoutines, tc, aid);
 	}
@@ -597,19 +616,6 @@ public	class DD_Version implements	Formatable
 
 		bootingDictionary.clearCaches();
 	}
-	
-	/**
- 	 * Make a catalog.
-	 *	@param	tc	TransactionController
-	 *	@exception StandardException  Standard Derby error policy.
-	 */
-	protected void makeSystemCatalog(TransactionController tc,
-									 TabInfoImpl ti)
-		throws StandardException
-	{
-		SchemaDescriptor sd = bootingDictionary.getSystemSchemaDescriptor();
-		bootingDictionary.makeCatalog(ti,sd,tc);
-	}
 
 	/**
 	  Remove the description of a System table from the data dictionary.
@@ -656,71 +662,6 @@ public	class DD_Version implements	Formatable
 		dropSystemCatalogDescription(tc,td);
 	}
 
-
-	/**
-	 * Populates a new system index from the base system table.
-	 *
-	 *	@param	tc						transaction controller
-	 *	@param	heapConglomerateNumber	identifies system table to Store
-	 *	@param	tabInfo					describes base system table
-	 *	@param	indexNumber				index to populate
-	 *
-	 *
-	 * @exception StandardException		Thrown on failure
-	 */
-	protected	void	fillIndex
-	(
-		TransactionController	tc,
-		long					heapConglomerateNumber,
-		TabInfoImpl					tabInfo,
-		int						indexNumber
-    )
-		throws StandardException
-	{
-		long						indexConglomerateNumber = tabInfo.getIndexConglomerate( indexNumber );
-		IndexRowGenerator			indexRowGenerator = tabInfo.getIndexRowGenerator( indexNumber );
-		CatalogRowFactory			rowFactory = tabInfo.getCatalogRowFactory();
-		ExecRow						heapRow = rowFactory.makeEmptyRow();
-		ExecIndexRow				indexableRow = indexRowGenerator.getIndexRowTemplate();
-
-		ScanController				heapScan =
-			tc.openScan(
-				heapConglomerateNumber,       // conglomerate to open
-				false,                          // don't hold open across commit
-				0,                              // for read
-                TransactionController.MODE_TABLE,
-                TransactionController.ISOLATION_REPEATABLE_READ,
-				(FormatableBitSet) null,                 // all fields as objects
-				null,                           // start position - first row
-				ScanController.GE,              // startSearchOperation
-				null,                           //scanQualifier,
-				null,                           //stop position-through last row
-				ScanController.GT);             // stopSearchOperation
-
-		RowLocation					heapLocation = 
-            heapScan.newRowLocationTemplate();
-
-		ConglomerateController		indexController = 
-			tc.openConglomerate( 
-				indexConglomerateNumber, 
-                false,
-				TransactionController.OPENMODE_FORUPDATE,
-				TransactionController.MODE_TABLE,
-				TransactionController.ISOLATION_REPEATABLE_READ);
-
-		while ( heapScan.fetchNext(heapRow.getRowArray()) )
-        {
- 			heapScan.fetchLocation( heapLocation );
-
-			indexRowGenerator.getIndexRow( heapRow, heapLocation, indexableRow, (FormatableBitSet) null );
-
-			indexController.insert(indexableRow.getRowArray());
-		}
-
-		indexController.close();
-		heapScan.close();
-	}
-
 	////////////////////////////////////////////////////////////////////////
 	//
 	//	FORMATABLE INTERFACE
@@ -786,23 +727,23 @@ public	class DD_Version implements	Formatable
 
 		jbmsVersion.getMinorVersion()*100 +jbmsVersion.getMaintVersion() + (jbmsVersion.isBeta() ? 0 : 1) + 2
 
-		5.0.22 => (0*100) + 22 + 2 =  24 - (5.0 has a unique major number)
-		5.1.2  => (1*100) + 2 + 2  = 104 - (5.1 has a unique major number) 
+		5.0.22 =&gt; (0*100) + 22 + 2 =  24 - (5.0 has a unique major number)
+		5.1.2  =&gt; (1*100) + 2 + 2  = 104 - (5.1 has a unique major number) 
 
 
 		With the switch to the four part scheme in 5.2, the maint number now is in increments of one million,
 		thus the above scheme could lead to duplicate numbers. Note that the major number may not change
 		when the minor external release changes, e.g. 5.2 and 5.3 could share a DD_Version major number.
 
-		5.2.1.100 => (2*100) + 1000100 + 2 = 1000302
-		5.3.1.0   => (3*100) + 1000000 + 2 = 1000302
+		5.2.1.100 =&gt; (2*100) + 1000100 + 2 = 1000302
+		5.3.1.0   =&gt; (3*100) + 1000000 + 2 = 1000302
 
 		
 
 	 */
 	private int getJBMSMinorVersionNumber() 
 	{
-		ProductVersionHolder jbmsVersion = Monitor.getMonitor().getEngineVersion();
+		ProductVersionHolder jbmsVersion = DataDictionaryImpl.getMonitor().getEngineVersion();
 
 		return jbmsVersion.getMinorVersion()*100 +jbmsVersion.getMaintVersion() + (jbmsVersion.isBeta() ? 0 : 1) + 2;
 	}
@@ -893,4 +834,29 @@ public	class DD_Version implements	Formatable
 		return true;
 	}
 
+    /**
+     * Privileged startup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  boolean isFullUpgrade( final Properties startParams, final String oldVersionInfo )
+        throws StandardException
+    {
+        try {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Boolean>()
+                 {
+                     public Boolean run()
+                         throws StandardException
+                     {
+                         return Monitor.isFullUpgrade( startParams, oldVersionInfo );
+                     }
+                 }
+                 ).booleanValue();
+        } catch (PrivilegedActionException pae)
+        {
+            throw StandardException.plainWrapException( pae );
+        }
+    }
+
 }
