@@ -22,14 +22,13 @@
 package	org.apache.derby.impl.sql.compile;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
-
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.compiler.MethodBuilder;
-import org.apache.derby.iapi.services.sanity.SanityManager;
-import org.apache.derby.iapi.sql.ResultColumnDescriptor;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.compile.Visitor;
 import org.apache.derby.iapi.sql.conn.Authorizer;
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
@@ -48,9 +47,9 @@ import org.apache.derby.impl.sql.CursorTableReference;
 
 public class CursorNode extends DMLStatementNode
 {
-	public final static int UNSPECIFIED = 0;
+    final static int UNSPECIFIED = 0;
 	public final static int READ_ONLY = 1;
-	public final static int UPDATE = 2;
+    final static int UPDATE = 2;
 
 	private String		name;
 	private OrderByList	orderByList;
@@ -65,14 +64,13 @@ public class CursorNode extends DMLStatementNode
 	** There can only be a list of updatable columns when FOR UPDATE
 	** is specified as part of the cursor specification.
 	*/
-	private List updatableColumns;
+	private List<String> updatableColumns;
 	private FromTable updateTable;
-	private ResultColumnDescriptor[]	targetColumnDescriptors;
     /**
      * List of {@code TableDescriptor}s for base tables whose associated
      * indexes should be checked for stale statistics.
      */
-    private ArrayList statsToUpdate;
+    private ArrayList<TableDescriptor> statsToUpdate;
     private boolean checkIndexStats;
 
 	//If cursor references session schema tables, save the list of those table names in savedObjects in compiler context
@@ -80,57 +78,70 @@ public class CursorNode extends DMLStatementNode
 	//At generate time, we save this position in activation for easy access to session table names list from compiler context
 	private int indexOfSessionTableNamesInSavedObjects = -1;
 
+    // true if this CursorNode is the driving left-join of a MERGE statement
+    private boolean forMergeStatement;
+    
 	/**
-	 * Initializer for a CursorNode
+     * Constructor for a CursorNode
 	 *
-	 * @param statementType	Type of statement (SELECT, UPDATE, INSERT)
-	 * @param resultSet	A ResultSetNode specifying the result set for
-	 *			the cursor
-	 * @param name		The name of the cursor, null if no name
-	 * @param orderByList	The order by list for the cursor, null if no
-	 *			order by list
-	 * @param offset The value of a <result offset clause> if present
-	 * @param fetchFirst The value of a <fetch first clause> if present
-	 * @param hasJDBClimitClause True if the offset/fetchFirst clauses come from JDBC limit/offset escape syntax
-	 * @param updateMode	The user-specified update mode for the cursor,
-	 *			for example, CursorNode.READ_ONLY
-	 * @param updatableColumns The list of updatable columns specified by
-	 *			the user in the FOR UPDATE clause, null if no
-	 *			updatable columns specified.  May only be
-	 *			provided if the updateMode parameter is
-	 *			CursorNode.UPDATE.
+     * @param statementType      Type of statement (SELECT, UPDATE, INSERT)
+     * @param resultSet          A ResultSetNode specifying the result set for
+     *                           the cursor
+     * @param name               The name of the cursor, null if no name
+     * @param orderByList        The order by list for the cursor, null if no
+     *                           order by list
+     * @param offset             The value of a <result offset clause> if
+     *                           present
+     * @param fetchFirst         The value of a <fetch first clause> if present
+     * @param hasJDBClimitClause True if the offset/fetchFirst clauses come
+     *                           from JDBC limit/offset escape syntax
+     * @param updateMode         The user-specified update mode for the cursor,
+     *                           for example, CursorNode.READ_ONLY
+     * @param updatableColumns   The array of updatable columns specified by
+     *                           the user in the FOR UPDATE clause, null if no
+     *                           updatable columns specified.  May only be
+     *                           provided if the updateMode parameter is
+     *                           CursorNode.UPDATE.
+     * @param forMergeStatement True if this cursor is the driving left-join of a MERGE statement
+     * @param cm                 The context manager
 	 */
-
-	public	void init(
-		Object statementType,
-		Object resultSet,
-		Object name,
-		Object orderByList,
-		Object offset,
-		Object fetchFirst,
-        Object hasJDBClimitClause,
-		Object updateMode,
-		Object updatableColumns)
+    CursorNode(String         statementType,
+               ResultSetNode  resultSet,
+               String         name,
+               OrderByList    orderByList,
+               ValueNode      offset,
+               ValueNode      fetchFirst,
+               boolean        hasJDBClimitClause,
+               int            updateMode,
+               String[]       updatableColumns,
+               boolean        forMergeStatement,
+               ContextManager cm)
 	{
-		init(resultSet);
-		this.name = (String) name;
-		this.statementType = (String) statementType;
-		this.orderByList = (OrderByList) orderByList;
-		this.offset = (ValueNode)offset;
-		this.fetchFirst = (ValueNode)fetchFirst;
-        this.hasJDBClimitClause = (hasJDBClimitClause == null) ? false : ((Boolean) hasJDBClimitClause).booleanValue();
-
-		this.updateMode = ((Integer) updateMode).intValue();
-		this.updatableColumns = (List) updatableColumns;
+        super(resultSet, cm);
+        this.name = name;
+        this.statementType = statementType;
+        this.orderByList = orderByList;
+        this.offset = offset;
+        this.fetchFirst = fetchFirst;
+        this.hasJDBClimitClause = hasJDBClimitClause;
+        this.updateMode = updateMode;
+        this.updatableColumns =
+                updatableColumns == null ?
+                null : Arrays.asList(updatableColumns);
+        this.forMergeStatement = forMergeStatement;
 
 		/*
 		** This is a sanity check and not an error since the parser
 		** controls setting updatableColumns and updateMode.
 		*/
-		if (SanityManager.DEBUG)
-		SanityManager.ASSERT(this.updatableColumns == null ||
-			this.updatableColumns.isEmpty() || this.updateMode == UPDATE,
-			"Can only have explicit updatable columns if update mode is UPDATE");
+        if (SanityManager.DEBUG) {
+            SanityManager.ASSERT(
+                    this.updatableColumns == null ||
+                    this.updatableColumns.isEmpty() ||
+                    this.updateMode == UPDATE,
+                    "Can only have explicit updatable columns if " +
+                        "update mode is UPDATE");
+        }
 	}
 
 	/**
@@ -139,7 +150,7 @@ public class CursorNode extends DMLStatementNode
 	 *
 	 * @return	This object as a String
 	 */
-
+    @Override
 	public String toString()
 	{
 		if (SanityManager.DEBUG)
@@ -154,7 +165,7 @@ public class CursorNode extends DMLStatementNode
 		}
 	}
 
-	public String statementToString()
+    String statementToString()
 	{
 		return statementType;
 	}
@@ -198,8 +209,8 @@ public class CursorNode extends DMLStatementNode
 	 *
 	 * @param depth		The depth of this node in the tree
 	 */
-
-	public void printSubNodes(int depth)
+    @Override
+    void printSubNodes(int depth)
 	{
 		if (SanityManager.DEBUG)
 		{
@@ -232,9 +243,14 @@ public class CursorNode extends DMLStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
+    @Override
 	public void bindStatement() throws StandardException
 	{
+        //
+        // Don't add USAGE privilege on user-defined types.
+        //
+        boolean wasSkippingTypePrivileges = getCompilerContext().skipTypePrivileges( true );
+            
 		DataDictionary				dataDictionary;
 
 		dataDictionary = getDataDictionary();
@@ -257,9 +273,8 @@ public class CursorNode extends DMLStatementNode
 
 		getCompilerContext().pushCurrentPrivType(getPrivType());
 		try {
-			FromList	fromList = (FromList) getNodeFactory().getNode(
-					C_NodeTypes.FROM_LIST,
-					getNodeFactory().doJoinOrderOptimization(),
+            FromList    fromList = new FromList(
+                    getOptimizerFactory().doJoinOrderOptimization(),
 					getContextManager());
 
 			/* Check for ? parameters directly under the ResultColums */
@@ -277,7 +292,7 @@ public class CursorNode extends DMLStatementNode
 
 			// Reject any XML values in the select list; JDBC doesn't
 			// define how we bind these out, so we don't allow it.
-			resultSet.rejectXMLValues();
+			if ( !forMergeStatement ) { resultSet.rejectXMLValues(); }
 
 			/* Verify that all underlying ResultSets reclaimed their FromList */
 			if (SanityManager.DEBUG) {
@@ -357,12 +372,10 @@ public class CursorNode extends DMLStatementNode
 			bindUpdateColumns(updateTable);
 
 			// If the target table is a FromBaseTable, mark the updatable
-			// columns.  (I can't think of a way that an updatable table
-			// could be anything but a FromBaseTable at this point, but
-			// it's better to be careful.
-			if (updateTable instanceof FromTable)
+            // columns.
+            if (updateTable != null)
 			{
-				((FromTable) updateTable).markUpdatableByCursor(updatableColumns);
+                updateTable.markUpdatableByCursor(updatableColumns);
 				//make sure that alongwith the FromTable, we keep other ResultSetLists
 				//in correct state too. ResultSetMetaData.isWritable looks at this to
 				//return the correct value.
@@ -378,12 +391,13 @@ public class CursorNode extends DMLStatementNode
 		{
 			//If this cursor has references to session schema tables, save the names of those tables into compiler context
 			//so they can be passed to execution phase.
-			ArrayList sessionSchemaTableNames = getSessionSchemaTableNamesForCursor();
+			ArrayList<String> sessionSchemaTableNames = getSessionSchemaTableNamesForCursor();
 			if (sessionSchemaTableNames != null)
 				indexOfSessionTableNamesInSavedObjects = getCompilerContext().addSavedObject(sessionSchemaTableNames);
 		}
 
-	}
+        getCompilerContext().skipTypePrivileges( wasSkippingTypePrivileges );
+    }
 
     /**
      * Collects table descriptors for base tables whose index statistics we
@@ -405,7 +419,7 @@ public class CursorNode extends DMLStatementNode
                 TableDescriptor td = fromTable.getTableDescriptor();
                 if (td.getTableType() == TableDescriptor.BASE_TABLE_TYPE) {
                     if (statsToUpdate == null) {
-                        statsToUpdate = new ArrayList();
+                        statsToUpdate = new ArrayList<TableDescriptor>();
                     }
                     statsToUpdate.add(td);
                 }
@@ -420,6 +434,7 @@ public class CursorNode extends DMLStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	public boolean referencesSessionSchema()
 		throws StandardException
 	{
@@ -430,13 +445,13 @@ public class CursorNode extends DMLStatementNode
 	//Check if this cursor references any session schema tables. If so, pass those names to execution phase through savedObjects
 	//This list will be used to check if there are any holdable cursors referencing temporary tables at commit time.
 	//If yes, then the data in those temporary tables should be preserved even if they are declared with ON COMMIT DELETE ROWS option
-	protected ArrayList getSessionSchemaTableNamesForCursor()
+	protected ArrayList<String> getSessionSchemaTableNamesForCursor()
 		throws StandardException
 	{
 		FromList fromList = resultSet.getFromList();
 		int fromListSize = fromList.size();
 		FromTable fromTable;
-		ArrayList sessionSchemaTableNames = null;
+		ArrayList<String> sessionSchemaTableNames = null;
 
 		for( int i = 0; i < fromListSize; i++)
 		{
@@ -444,7 +459,7 @@ public class CursorNode extends DMLStatementNode
 			if (fromTable instanceof FromBaseTable && isSessionSchema(fromTable.getTableDescriptor().getSchemaDescriptor()))
 			{
 				if (sessionSchemaTableNames == null)
-					sessionSchemaTableNames = new ArrayList();
+					sessionSchemaTableNames = new ArrayList<String>();
 				sessionSchemaTableNames.add(fromTable.getTableName().getTableName());
 			}
 		}
@@ -471,7 +486,7 @@ public class CursorNode extends DMLStatementNode
 	 *     <LI>if it does not have exactly 1 table in its FROM list;
 	 *         0 tables would occur if we ever support a SELECT without a
 	 *         FROM e.g., for generating a row without an underlying table
-	 *         (like what we do for an INSERT of a VALUES list); >1 tables
+	 *         (like what we do for an INSERT of a VALUES list); &gt;1 tables
 	 *         occurs when joins are in the tree.
 	 *     <LI>if the table in its FROM list is not a base table (REMIND
 	 *         when views/from subqueries are added, this should be relaxed to
@@ -533,17 +548,9 @@ public class CursorNode extends DMLStatementNode
 		if (updateTable.markAsCursorTargetTable()) {
 			/* Cursor is updatable - remember to generate the position code */
 			needTarget = true;
-
-			/* We must generate the target column list at bind time
-			 * because the optimizer may transform the FromBaseTable from
-			 * a table scan into an index scan.
-			 */
-			genTargetResultColList();
 		}
 
 
-
-
 		return UPDATE;
 	}
 
@@ -562,9 +569,10 @@ public class CursorNode extends DMLStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
+    @Override
 	public void optimizeStatement() throws StandardException
 	{
+        resultSet.pushQueryExpressionSuffix();
 		// Push the order by list down to the ResultSet
 		if (orderByList != null)
 		{
@@ -579,7 +587,9 @@ public class CursorNode extends DMLStatementNode
 			orderByList = null;
 		}
 
-        resultSet.pushOffsetFetchFirst( offset, fetchFirst, hasJDBClimitClause );
+        resultSet.pushOffsetFetchFirst(offset, fetchFirst, hasJDBClimitClause);
+        offset = null;
+        fetchFirst = null;
 
         super.optimizeStatement();
 
@@ -593,7 +603,7 @@ public class CursorNode extends DMLStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	 
+    @Override
 	int activationKind()
 	{
 		return NEED_CURSOR_ACTIVATION;
@@ -607,9 +617,8 @@ public class CursorNode extends DMLStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public void generate(ActivationClassBuilder acb,
-								MethodBuilder mb) throws StandardException
+    @Override
+    void generate(ActivationClassBuilder acb, MethodBuilder mb) throws StandardException
 	{
 		if (indexOfSessionTableNamesInSavedObjects != -1 ) //if this cursor references session schema tables, do following
 		{
@@ -647,105 +656,29 @@ public class CursorNode extends DMLStatementNode
 
 	// class interface
 
-	public String getUpdateBaseTableName() 
+    String getUpdateBaseTableName()
 	{
 		return (updateTable == null) ? null : updateTable.getBaseTableName();
 	}
 
-	public String getUpdateExposedTableName() 
+    String getUpdateExposedTableName()
 		throws StandardException
 	{
 		return (updateTable == null) ? null : updateTable.getExposedName();
 	}
 
-	public String getUpdateSchemaName() 
+    String getUpdateSchemaName()
 		throws StandardException
 	{
 		//we need to use the base table for the schema name
 		return (updateTable == null) ? null : ((FromBaseTable)updateTable).getTableNameField().getSchemaName();
 	}
 
-	public int getUpdateMode()
+    int getUpdateMode()
 	{
 		return updateMode;
 	}
 
-	/**
-	 * Return String[] of names from the FOR UPDATE OF List
-	 *
-	 * @return	String[] of names from the FOR UPDATE OF list.
-	 */
-	private String[] getUpdatableColumns()
-	{
-		return (updatableColumns == null) ?
-				(String[])null :
-				getUpdateColumnNames();
-	}
-
-	/**
-		Positioned update needs to know what the target result set
-		looks like. This is generated from the UpdateColumnList
-		available for the cursor, to describe the rows coming from
-		the target result set under the cursor. This result set contains
-		a superset of the updatable columns; the caller must verify that
-		only those listed in the FOR UPDATE clause are used.
-
-		@return a result column list containing a description of
-		the target table (this may contain non-updatable columns).
-	 * @exception StandardException		Thrown on error
-	 */
-	private ResultColumnDescriptor[] genTargetResultColList()
-		throws StandardException
-	{
-		ResultColumnList newList;
-
-		/*
-		   updateTable holds the FromTable that is the target.
-		   copy its ResultColumnList, making BaseColumn references
-		   for use in the CurrentOfNode, which behaves as if it had
-		   base columns for the statement it is in.
-
-			updateTable is null if the cursor is not updatable.
-		 */
-		if (updateTable == null) return null;
-
-		if (targetColumnDescriptors != null) return targetColumnDescriptors;
-
-		newList = (ResultColumnList) getNodeFactory().getNode(
-										C_NodeTypes.RESULT_COLUMN_LIST,
-										getContextManager());
-		ResultColumnList rcl = updateTable.getResultColumns();
-		int rclSize = rcl.size();
-		for (int index = 0; index < rclSize; index++)
-		{
-			ResultColumn origCol, newCol;
-			ValueNode newNode;
-
-			origCol = (ResultColumn) rcl.elementAt(index);
-
-			// Build a ResultColumn/BaseColumnNode pair for the column
-			newNode = (ValueNode) getNodeFactory().getNode(
-							C_NodeTypes.BASE_COLUMN_NODE,
-							origCol.getName(),
-							makeTableName(origCol.getSchemaName(),
-										  origCol.getTableName()),								
-							origCol.getTypeServices(),
-							getContextManager());
-			newCol = (ResultColumn) getNodeFactory().getNode(
-									C_NodeTypes.RESULT_COLUMN,
-									origCol.columnDescriptor,
-									newNode,
-									getContextManager());
-
-			/* Build the ResultColumnList to return */
-			newList.addResultColumn(newCol);
-		}
-
-		// we save the result so we only do this once
-		targetColumnDescriptors = newList.makeResultDescriptors();
-		return targetColumnDescriptors;
-	}
-
 	/**
 	 * Returns whether or not this Statement requires a set/clear savepoint
 	 * around its execution.  The following statement "types" do not require them:
@@ -754,6 +687,7 @@ public class CursorNode extends DMLStatementNode
 	 *
 	 * @return boolean	Whether or not this Statement requires a set/clear savepoint
 	 */
+    @Override
 	public boolean needsSavepoint()
 	{
 		return false;
@@ -767,6 +701,7 @@ public class CursorNode extends DMLStatementNode
 	 * @return	the cursor info
 	 * @exception StandardException thrown if generation fails
 	 */
+    @Override
 	public Object getCursorInfo()
 		throws StandardException
 	{
@@ -778,8 +713,7 @@ public class CursorNode extends DMLStatementNode
 										getUpdateExposedTableName(),
 										getUpdateBaseTableName(),
 										getUpdateSchemaName()),
-								genTargetResultColList(),
-								getUpdatableColumns());
+                                updatableColumns);
 	}
 
 	/**
@@ -799,51 +733,45 @@ public class CursorNode extends DMLStatementNode
 		int size = updatableColumns.size();
 		TableDescriptor tableDescriptor;
 		String columnName;
-		ResultColumnList rcls = resultSet.getResultColumns();
+        ResultColumnList rcl = resultSet.getResultColumns();
 
 		for (int index = 0; index < size; index++)
 		{
-		    columnName = (String) updatableColumns.get(index);
+		    columnName = updatableColumns.get(index);
 		    tableDescriptor = targetTable.getTableDescriptor();
 		    if ( tableDescriptor.getColumnDescriptor(columnName) == null)
 		    {
 					throw StandardException.newException(SQLState.LANG_COLUMN_NOT_FOUND, columnName);
 		    }
 
-		    ResultColumn rc;
-		    //make sure that we are not using correlation names for updatable columns. 
-		    //eg select c11 as col1, 2, c13 as col3 from t1 for update of c11, c12
-		    //In the eg above, correlation name for c11 will cause exception because Derby does not support correlation name for updatable columns
-		    //But correlation name for c13 is ok because it is a read only column
-		    for (int rclsIndex = 0; rclsIndex < rcls.size(); rclsIndex++) {//look through each column in the resultset for cursor
-					rc = ((ResultColumn) rcls.elementAt(rclsIndex));
-					if (rc.getSourceTableName() == null) //continue to look at the next column because this is derived column in the select list
-					continue;
-					if (rc.getExpression() != null && rc.getExpression().getColumnName().equals(columnName) &&  !rc.getName().equals(columnName)) {
-					throw StandardException.newException(SQLState.LANG_CORRELATION_NAME_FOR_UPDATABLE_COLUMN_DISALLOWED_IN_CURSOR, columnName);
-					}
+            // Make sure that we are not using correlation names for updatable
+            // columns.
+            //
+            // Example:
+            //     select c11 as col1, 2, c13 as col3 from t1
+            //         for update of c11, c12
+            //
+            // The correlation name for c11 will cause an exception because
+            // Derby does not support correlation name for updatable
+            // columns. However, a correlation name for c13 is ok because it is
+            // a read only column.
+
+            for (ResultColumn rc : rcl) {
+                // Look through each column in the resultset for cursor.
+                if (rc.getSourceTableName() == null) {
+                    // Continue to look at the next column because this is
+                    // a derived column in the select list.
+                    continue;
+                }
+
+                if (rc.getExpression() != null && rc.getExpression().getColumnName().equals(columnName) &&  !rc.getName().equals(columnName)) {
+                    throw StandardException.newException(SQLState.LANG_CORRELATION_NAME_FOR_UPDATABLE_COLUMN_DISALLOWED_IN_CURSOR, columnName);
+                }
 		    }
 		}
 	}
 
-	/**
-	 * Get an array of strings for each updatable column
-	 * in this list.
-	 *
-	 * @return an array of strings
-	 */
-	private String[] getUpdateColumnNames()
-	{
-		int size = updatableColumns.size();
-		if (size == 0)
-		{
-			return (String[])null;
-		}
-
-        return (String[]) updatableColumns.toArray(new String[size]);
-	}
-	
-	public String getXML()
+    String getXML()
 	{
 		return null;
 	}
@@ -856,6 +784,7 @@ public class CursorNode extends DMLStatementNode
      * @throws StandardException if accessing the index descriptors of a base
      *      table fails
      */
+    @Override
     public TableDescriptor[] updateIndexStatisticsFor()
             throws StandardException {
         if (!checkIndexStats || statsToUpdate == null) {
@@ -865,7 +794,7 @@ public class CursorNode extends DMLStatementNode
         // Iterate backwards to remove elements, chances are high the stats are
         // mostly up-to-date (minor performance optimization to avoid copy).
         for (int i=statsToUpdate.size() -1; i >= 0; i--) {
-            TableDescriptor td = (TableDescriptor)statsToUpdate.get(i);
+            TableDescriptor td = statsToUpdate.get(i);
             if (td.getAndClearIndexStatsIsUpToDate()) {
                 statsToUpdate.remove(i);
             }
@@ -887,6 +816,7 @@ public class CursorNode extends DMLStatementNode
 	 *
 	 * @exception StandardException on error
 	 */
+    @Override
 	void acceptChildren(Visitor v)
 		throws StandardException
 	{
