@@ -21,51 +21,39 @@
 
 package	org.apache.derby.impl.sql.compile;
 
-import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.error.StandardException;
-
-import org.apache.derby.iapi.sql.conn.Authorizer;
-import org.apache.derby.iapi.sql.dictionary.DataDictionary;
-import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
-import org.apache.derby.iapi.sql.dictionary.GenericDescriptorList;
-import org.apache.derby.iapi.sql.dictionary.ColumnDescriptor;
-import org.apache.derby.iapi.sql.dictionary.ColumnDescriptorList;
-import org.apache.derby.iapi.sql.dictionary.TriggerDescriptor;
-
-import org.apache.derby.iapi.sql.StatementType;
-
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-import org.apache.derby.iapi.reference.ClassName;
-
-import org.apache.derby.iapi.sql.execute.ConstantAction;
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
-import org.apache.derby.iapi.services.compiler.MethodBuilder;
-
-import org.apache.derby.iapi.store.access.StaticCompiledOpenConglomInfo;
-import org.apache.derby.iapi.store.access.TransactionController;
-
-import org.apache.derby.vti.DeferModification;
-
-import org.apache.derby.catalog.UUID;
-import org.apache.derby.iapi.services.io.FormatableBitSet;
-
-import org.apache.derby.impl.sql.execute.FKInfo;
-
 import java.lang.reflect.Modifier;
-import org.apache.derby.iapi.services.classfile.VMOpcode;
-import org.apache.derby.iapi.services.io.FormatableProperties;
-
 import java.util.ArrayList;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
-import org.apache.derby.iapi.sql.compile.NodeFactory;
-import org.apache.derby.iapi.util.ReuseFactory;
-import org.apache.derby.iapi.sql.ResultDescription;
+import org.apache.derby.catalog.UUID;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.ClassName;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.classfile.VMOpcode;
 import org.apache.derby.iapi.services.compiler.LocalField;
+import org.apache.derby.iapi.services.compiler.MethodBuilder;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.io.FormatableBitSet;
+import org.apache.derby.iapi.services.io.FormatableProperties;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.sql.ResultDescription;
+import org.apache.derby.iapi.sql.StatementType;
+import org.apache.derby.iapi.sql.compile.CompilerContext;
+import org.apache.derby.iapi.sql.compile.IgnoreFilter;
+import org.apache.derby.iapi.sql.compile.ScopeFilter;
+import org.apache.derby.iapi.sql.conn.Authorizer;
+import org.apache.derby.iapi.sql.dictionary.ColumnDescriptor;
+import org.apache.derby.iapi.sql.dictionary.ColumnDescriptorList;
+import org.apache.derby.iapi.sql.dictionary.ConglomerateDescriptor;
+import org.apache.derby.iapi.sql.dictionary.DataDictionary;
+import org.apache.derby.iapi.sql.dictionary.TriggerDescriptorList;
+import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
+import org.apache.derby.iapi.sql.dictionary.TriggerDescriptor;
+import org.apache.derby.iapi.sql.execute.ConstantAction;
+import org.apache.derby.iapi.store.access.StaticCompiledOpenConglomInfo;
+import org.apache.derby.iapi.store.access.TransactionController;
+import org.apache.derby.vti.DeferModification;
 
 
 /**
@@ -77,38 +65,44 @@ import org.apache.derby.iapi.services.compiler.LocalField;
  * the named cursor.
  *
  */
-
-public class DeleteNode extends DMLModStatementNode
+class DeleteNode extends DMLModStatementNode
 {
 	/* Column name for the RowLocation column in the ResultSet */
 	private static final String COLUMNNAME = "###RowLocationToDelete";
 
 	/* Filled in by bind. */
-	protected boolean				deferred;
-	protected FromTable				targetTable;
-	protected FKInfo				fkInfo;
-	protected FormatableBitSet readColsBitSet;
+    private boolean deferred;
+    private FromTable targetTable;
+    private FormatableBitSet readColsBitSet;
 
 	private ConstantAction[] dependentConstantActions;
 	private boolean cascadeDelete;
 	private StatementNode[] dependentNodes;
 
 	/**
-	 * Initializer for a DeleteNode.
+     * Constructor for a DeleteNode.
 	 *
 	 * @param targetTableName	The name of the table to delete from
 	 * @param queryExpression	The query expression that will generate
 	 *				the rows to delete from the given table
+     * @param matchingClause   Non-null if this DML is part of a MATCHED clause of a MERGE statement.
+     * @param cm                The context manager
 	 */
 
-	public void init(Object targetTableName,
-					  Object queryExpression)
-	{
-		super.init(queryExpression);
-		this.targetTableName = (TableName) targetTableName;
+    DeleteNode
+        (
+         TableName targetTableName,
+         ResultSetNode queryExpression,
+         MatchingClauseNode matchingClause,
+         ContextManager cm
+         )
+    {
+        super( queryExpression, matchingClause, cm );
+        this.targetTableName = targetTableName;
 	}
 
-	public String statementToString()
+    @Override
+    String statementToString()
 	{
 		return "DELETE";
 	}
@@ -128,24 +122,33 @@ public class DeleteNode extends DMLModStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
+    @Override
 	public void bindStatement() throws StandardException
 	{
 		// We just need select privilege on the where clause tables
 		getCompilerContext().pushCurrentPrivType( Authorizer.SELECT_PRIV);
 		try
 		{
-			FromList	fromList = (FromList) getNodeFactory().getNode(
-									C_NodeTypes.FROM_LIST,
-									getNodeFactory().doJoinOrderOptimization(),
-									getContextManager());
-			ResultColumn				rowLocationColumn = null;
+            FromList fromList = new FromList(
+                    getOptimizerFactory().doJoinOrderOptimization(),
+                    getContextManager());
+
+            ResultColumn                rowLocationColumn = null;
 			CurrentRowLocationNode		rowLocationNode;
 			TableName					cursorTargetTableName = null;
 			CurrentOfNode       		currentOfNode = null;
-		
+
+            //
+            // Don't add privilege requirements for the UDT types of columns.
+            // The compiler will attempt to add these when generating the full column list during
+            // binding of the tables.
+            //
+            IgnoreFilter    ignorePermissions = new IgnoreFilter();
+            getCompilerContext().addPrivilegeFilter( ignorePermissions );
+            
 			DataDictionary dataDictionary = getDataDictionary();
-			super.bindTables(dataDictionary);
+            // for DELETE clause of a MERGE statement, the tables have already been bound
+			if ( !inMatchingClause() ) { super.bindTables(dataDictionary); }
 
 			// wait to bind named target table until the underlying
 			// cursor is bound, so that we can get it from the
@@ -156,14 +159,14 @@ public class DeleteNode extends DMLModStatementNode
 				SanityManager.ASSERT(resultSet != null && resultSet instanceof SelectNode,
 				"Delete must have a select result set");
 
-			SelectNode sel;
-			sel = (SelectNode)resultSet;
+            SelectNode sel = (SelectNode)resultSet;
 			targetTable = (FromTable) sel.fromList.elementAt(0);
 			if (targetTable instanceof CurrentOfNode)
 			{
 				currentOfNode = (CurrentOfNode) targetTable;
 
-				cursorTargetTableName = currentOfNode.getBaseCursorTargetTableName();
+				cursorTargetTableName = inMatchingClause() ?
+                    targetTableName : currentOfNode.getBaseCursorTargetTableName();
 				// instead of an assert, we might say the cursor is not updatable.
 				if (SanityManager.DEBUG)
 					SanityManager.ASSERT(cursorTargetTableName != null);
@@ -205,9 +208,11 @@ public class DeleteNode extends DMLModStatementNode
 			verifyTargetTable();
 
 			/* Generate a select list for the ResultSetNode - CurrentRowLocation(). */
-			if (SanityManager.DEBUG)
-				SanityManager.ASSERT((resultSet.resultColumns == null),
+			if ( SanityManager.DEBUG )
+            {
+				SanityManager.ASSERT((resultSet.getResultColumns() == null),
 							  "resultColumns is expected to be null until bind time");
+            }
 
 
 			if (targetTable instanceof FromVTI)
@@ -221,12 +226,14 @@ public class DeleteNode extends DMLModStatementNode
 			}
 			else
 			{
+            
 				/*
 				** Start off assuming no columns from the base table
 				** are needed in the rcl.
 				*/
 
-				resultColumnList = new ResultColumnList();
+                resultColumnList =
+                        new ResultColumnList(getContextManager());
 
 				FromBaseTable fbt = getResultColumnList(resultColumnList);
 
@@ -255,15 +262,12 @@ public class DeleteNode extends DMLModStatementNode
 				}
 
 				/* Generate the RowLocation column */
-				rowLocationNode = (CurrentRowLocationNode) getNodeFactory().getNode(
-										C_NodeTypes.CURRENT_ROW_LOCATION_NODE,
-										getContextManager());
-				rowLocationColumn =
-					(ResultColumn) getNodeFactory().getNode(
-									C_NodeTypes.RESULT_COLUMN,
-									COLUMNNAME,
-									rowLocationNode,
-									getContextManager());
+                rowLocationNode =
+                        new CurrentRowLocationNode(getContextManager());
+                rowLocationColumn =
+                        new ResultColumn(COLUMNNAME,
+                                         rowLocationNode,
+                                         getContextManager());
 				rowLocationColumn.markGenerated();
 
 				/* Append to the ResultColumnList */
@@ -272,12 +276,26 @@ public class DeleteNode extends DMLModStatementNode
 				/* Force the added columns to take on the table's correlation name, if any */
 				correlateAddedColumns( resultColumnList, targetTable );
 			
-				/* Set the new result column list in the result set */
+                /* Add the new result columns to the driving result set */
+                ResultColumnList    originalRCL = resultSet.getResultColumns();
+                if ( originalRCL != null )
+                {
+                    originalRCL.appendResultColumns( resultColumnList, false );
+                    resultColumnList = originalRCL;
+                }
 				resultSet.setResultColumns(resultColumnList);
 			}
 
+            // done excluding column types from privilege checking
+            getCompilerContext().removePrivilegeFilter( ignorePermissions );
+
 			/* Bind the expressions before the ResultColumns are bound */
+
+            // only add privileges when we're inside the WHERE clause
+            ScopeFilter scopeFilter = new ScopeFilter( getCompilerContext(), CompilerContext.WHERE_SCOPE, 1 );
+            getCompilerContext().addPrivilegeFilter( scopeFilter );
 			super.bindExpressions();
+            getCompilerContext().removePrivilegeFilter( scopeFilter );
 
 			/* Bind untyped nulls directly under the result columns */
 			resultSet.getResultColumns().
@@ -287,16 +305,15 @@ public class DeleteNode extends DMLModStatementNode
 			{
 				/* Bind the new ResultColumn */
 				rowLocationColumn.bindResultColumnToExpression();
-
 				bindConstraints(dataDictionary,
-							getNodeFactory(),
-							targetTableDescriptor,
-							null,
-							resultColumnList,
-							(int[]) null,
-							readColsBitSet,
-							false,
-							true);  /* we alway include triggers in core language */
+                        getOptimizerFactory(),
+                        targetTableDescriptor,
+                        null,
+                        resultColumnList,
+                        (int[]) null,
+                        readColsBitSet,
+                        true, // we alway include triggers in core language
+                        new boolean[1]); // dummy
 
 				/* If the target table is also a source table, then
 			 	* the delete will have to be in deferred mode
@@ -318,7 +335,6 @@ public class DeleteNode extends DMLModStatementNode
                                                   null,
                                                   sel.getWhereClause());
 			}
-        	sel = null; // done with sel
 
 			/* Verify that all underlying ResultSets reclaimed their FromList */
 			if (SanityManager.DEBUG)
@@ -338,7 +354,7 @@ public class DeleteNode extends DMLModStatementNode
 
 				if(!isDependentTable){
 					//graph node
-                    dependentTables = new HashSet();
+                    dependentTables = new HashSet<String>();
 				}
 
 				/*Check whether the current target has already been explored.
@@ -370,12 +386,11 @@ public class DeleteNode extends DMLModStatementNode
 
 				}
 			}
-			if (isPrivilegeCollectionRequired())
-			{
-				getCompilerContext().pushCurrentPrivType( getPrivType());
-				getCompilerContext().addRequiredTablePriv( targetTableDescriptor);
-				getCompilerContext().popCurrentPrivType();
-			}
+
+            // add need for DELETE privilege on the target table
+            getCompilerContext().pushCurrentPrivType( getPrivType());
+            getCompilerContext().addRequiredTablePriv( targetTableDescriptor);
+            getCompilerContext().popCurrentPrivType();
 		}
 		finally
 		{
@@ -383,6 +398,7 @@ public class DeleteNode extends DMLModStatementNode
 		}
 	} // end of bind
 
+    @Override
 	int getPrivType()
 	{
 		return Authorizer.DELETE_PRIV;
@@ -395,6 +411,7 @@ public class DeleteNode extends DMLModStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	public boolean referencesSessionSchema()
 		throws StandardException
 	{
@@ -407,14 +424,15 @@ public class DeleteNode extends DMLModStatementNode
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
-	public ConstantAction	makeConstantAction() throws StandardException
+    @Override
+    public ConstantAction makeConstantAction() throws StandardException
 	{
 
 		/* Different constant actions for base tables and updatable VTIs */
 		if (targetTableDescriptor != null)
 		{
 			// Base table
-			int lockMode = resultSet.updateTargetLockMode();
+            int lckMode = resultSet.updateTargetLockMode();
 			long heapConglomId = targetTableDescriptor.getHeapConglomerateId();
 			TransactionController tc = getLanguageConnectionContext().getTransactionCompile();
 			StaticCompiledOpenConglomInfo[] indexSCOCIs = 
@@ -431,7 +449,7 @@ public class DeleteNode extends DMLModStatementNode
 			*/
 			if (targetTableDescriptor.getLockGranularity() == TableDescriptor.TABLE_LOCK_GRANULARITY)
 			{
-				lockMode = TransactionController.MODE_TABLE;
+                lckMode = TransactionController.MODE_TABLE;
 			}
 
 			ResultDescription resultDescription = null;
@@ -454,7 +472,7 @@ public class DeleteNode extends DMLModStatementNode
 				  deferred,
 				  false,
 				  targetTableDescriptor.getUUID(),
-				  lockMode,
+                  lckMode,
 				  null, null, null, 0, null, null, 
 				  resultDescription,
 				  getFKInfo(), 
@@ -467,7 +485,8 @@ public class DeleteNode extends DMLModStatementNode
 					  readColsBitSet.getNumBitsSet(),			
 				  (UUID) null,
 				  resultSet.isOneRowResultSet(),
-				  dependentConstantActions);
+				  dependentConstantActions,
+                  inMatchingClause());
 		}
 		else
 		{
@@ -500,11 +519,10 @@ public class DeleteNode extends DMLModStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void generate(ActivationClassBuilder acb,
-								MethodBuilder mb)
+    @Override
+    void generate(ActivationClassBuilder acb, MethodBuilder mb)
 							throws StandardException
 	{
-
 		// If the DML is on the temporary table, generate the code to
 		// mark temporary table as modified in the current UOW. After
 		// DERBY-827 this must be done in execute() since
@@ -517,7 +535,16 @@ public class DeleteNode extends DMLModStatementNode
 
 		acb.pushGetResultSetFactoryExpression(mb); 
 		acb.newRowLocationScanResultSetName();
-		resultSet.generate(acb, mb); // arg 1
+
+        // arg 1
+        if ( inMatchingClause() )
+        {
+            matchingClause.generateResultSetField( acb, mb );
+        }
+        else
+        {
+            resultSet.generate( acb, mb );
+        }
 
 		String resultSetGetter;
 		int argCount;
@@ -579,7 +606,6 @@ public class DeleteNode extends DMLModStatementNode
 			mb.setField(arrayField);
 			for(int index=0 ; index <  dependentNodes.length ; index++)
 			{
-
 				dependentNodes[index].setRefActionInfo(fkIndexConglomNumbers[index],
 													   fkColArrays[index],
 													   parentResultSetId,
@@ -655,6 +681,7 @@ public class DeleteNode extends DMLModStatementNode
 	 *
 	 * @return the type of statement
 	 */
+    @Override
 	protected final int getStatementType()
 	{
 		return StatementType.DELETE;
@@ -690,12 +717,13 @@ public class DeleteNode extends DMLModStatementNode
 		boolean[]	needsDeferredProcessing = new boolean[1];
 		needsDeferredProcessing[0] = requiresDeferredProcessing();
 
-		ArrayList conglomVector = new ArrayList();
-		relevantTriggers = new GenericDescriptorList();
+        ArrayList<ConglomerateDescriptor> conglomerates = new ArrayList<ConglomerateDescriptor>();
+        relevantTriggers = new TriggerDescriptorList();
 
-		FormatableBitSet	columnMap = DeleteNode.getDeleteReadMap(baseTable,conglomVector, relevantTriggers, needsDeferredProcessing);
-		
-		markAffectedIndexes( conglomVector );
+        FormatableBitSet columnMap = DeleteNode.getDeleteReadMap(baseTable,
+                conglomerates, relevantTriggers, needsDeferredProcessing);
+
+        markAffectedIndexes(conglomerates);
 
 		adjustDeferredFlag( needsDeferredProcessing[0] );
 
@@ -740,21 +768,19 @@ public class DeleteNode extends DMLModStatementNode
     private DeleteNode getEmptyDeleteNode(String schemaName, String targetTableName)
         throws StandardException
     {
-
         ValueNode whereClause = null;
 
-        TableName tableName = new TableName();
-        tableName.init(schemaName , targetTableName);
+        TableName tableName =
+            new TableName(schemaName , targetTableName, getContextManager());
 
-        NodeFactory nodeFactory = getNodeFactory();
-        FromList   fromList = (FromList) nodeFactory.getNode(C_NodeTypes.FROM_LIST, getContextManager());
-        FromTable fromTable = (FromTable) nodeFactory.getNode(
-                                                    C_NodeTypes.FROM_BASE_TABLE,
-                                                    tableName,
-                                                    null,
-                                                    ReuseFactory.getInteger(FromBaseTable.DELETE),
-                                                    null,
-                                                    getContextManager());
+        FromList fromList = new FromList(getContextManager());
+
+        FromTable fromTable = new FromBaseTable(
+                tableName,
+                null,
+                FromBaseTable.DELETE,
+                null,
+                getContextManager());
 
 		//we would like to use references index & table scan instead of 
 		//what optimizer says for the dependent table scan.
@@ -763,23 +789,16 @@ public class DeleteNode extends DMLModStatementNode
 		((FromBaseTable) fromTable).setTableProperties(targetProperties);
 
         fromList.addFromTable(fromTable);
-        SelectNode resultSet = (SelectNode) nodeFactory.getNode(
-                                                     C_NodeTypes.SELECT_NODE,
-                                                     null,
-                                                     null,   /* AGGREGATE list */
-                                                     fromList, /* FROM list */
-                                                     whereClause, /* WHERE clause */
-                                                     null, /* GROUP BY list */
-                                                     null, /* having clause */
-													 null, /* windows */
-													 getContextManager());
-
-        return (DeleteNode) nodeFactory.getNode(
-                                                    C_NodeTypes.DELETE_NODE,
-                                                    tableName,
-                                                    resultSet,
-                                                    getContextManager());
+        SelectNode rs = new SelectNode(null,
+                                       fromList, /* FROM list */
+                                       whereClause, /* WHERE clause */
+                                       null, /* GROUP BY list */
+                                       null, /* having clause */
+                                       null, /* windows */
+                                       null, /* optimizer override plan */
+                                       getContextManager());
 
+        return new DeleteNode(tableName, rs, null, getContextManager());
     }
 
 
@@ -792,18 +811,17 @@ public class DeleteNode extends DMLModStatementNode
 
         ValueNode whereClause = null;
 
-        TableName tableName = new TableName();
-        tableName.init(schemaName , targetTableName);
+        TableName tableName =
+            new TableName(schemaName , targetTableName, getContextManager());
 
-        NodeFactory nodeFactory = getNodeFactory();
-        FromList   fromList = (FromList) nodeFactory.getNode(C_NodeTypes.FROM_LIST, getContextManager());
-        FromTable fromTable = (FromTable) nodeFactory.getNode(
-                                                    C_NodeTypes.FROM_BASE_TABLE,
-                                                    tableName,
-                                                    null,
-                                                    ReuseFactory.getInteger(FromBaseTable.DELETE),
-                                                    null,
-                                                    getContextManager());
+        FromList fromList = new FromList(getContextManager());
+
+        FromTable fromTable = new FromBaseTable(
+                tableName,
+                null,
+                FromBaseTable.DELETE,
+                null,
+                getContextManager());
 
 
 		//we would like to use references index & table scan instead of 
@@ -814,53 +832,38 @@ public class DeleteNode extends DMLModStatementNode
 
         fromList.addFromTable(fromTable);
 
-        SelectNode resultSet = (SelectNode) nodeFactory.getNode(
-                                                     C_NodeTypes.SELECT_NODE,
-                                                     getSetClause(tableName, cdl),
-                                                     null,   /* AGGREGATE list */
-                                                     fromList, /* FROM list */
-                                                     whereClause, /* WHERE clause */
-                                                     null, /* GROUP BY list */
-													 null, /* having clause */
-													 null, /* windows */
-                                                     getContextManager());
-
-        return (UpdateNode) nodeFactory.getNode(
-                                                    C_NodeTypes.UPDATE_NODE,
-                                                    tableName,
-                                                    resultSet,
-                                                    getContextManager());
+        SelectNode sn = new SelectNode(getSetClause(cdl),
+                                              fromList, /* FROM list */
+                                              whereClause, /* WHERE clause */
+                                              null, /* GROUP BY list */
+                                              null, /* having clause */
+                                              null, /* windows */
+                                              null, /* optimizer override plan */
+                                              getContextManager());
 
+        return new UpdateNode(tableName, sn, null, getContextManager());
     }
 
 
  
-	private ResultColumnList getSetClause(TableName tabName,
-										  ColumnDescriptorList cdl)
+    private ResultColumnList getSetClause(ColumnDescriptorList cdl)
 		throws StandardException
 	{
 		ResultColumn resultColumn;
 		ValueNode	 valueNode;
 
-		NodeFactory nodeFactory = getNodeFactory();
-		ResultColumnList	columnList = (ResultColumnList) nodeFactory.getNode(
-												C_NodeTypes.RESULT_COLUMN_LIST,
-												getContextManager());
+        ResultColumnList columnList = new ResultColumnList(getContextManager());
 
-		valueNode =  (ValueNode) nodeFactory.getNode(C_NodeTypes.UNTYPED_NULL_CONSTANT_NODE,
-															 getContextManager());
+        valueNode = new UntypedNullConstantNode(getContextManager());
 		for(int index =0 ; index < cdl.size() ; index++)
 		{
-			ColumnDescriptor cd = (ColumnDescriptor) cdl.elementAt(index);
+            ColumnDescriptor cd = cdl.elementAt(index);
 			//only columns that are nullable need to be set to 'null' for ON
 			//DELETE SET NULL
 			if((cd.getType()).isNullable())
 			{
-				resultColumn = (ResultColumn) nodeFactory.getNode(
-   									    C_NodeTypes.RESULT_COLUMN,
-										cd,
-										valueNode,
-										getContextManager());
+                resultColumn =
+                        new ResultColumn(cd, valueNode, getContextManager());
 
 				columnList.addResultColumn(resultColumn);
 			}
@@ -868,9 +871,13 @@ public class DeleteNode extends DMLModStatementNode
 		return columnList;
 	}
 
-
+    @Override
 	public void optimizeStatement() throws StandardException
 	{
+        // Don't add any more permissions during pre-processing
+        IgnoreFilter    ignorePermissions = new IgnoreFilter();
+        getCompilerContext().addPrivilegeFilter( ignorePermissions );
+        
 		if(cascadeDelete)
 		{
 			for(int index=0 ; index < dependentNodes.length ; index++)
@@ -879,7 +886,11 @@ public class DeleteNode extends DMLModStatementNode
 			}
 		}
 
-		super.optimizeStatement();
+        super.optimizeStatement();
+
+        // allow more permissions to be added in case we're just one action
+        // of a MERGE statement
+        getCompilerContext().removePrivilegeFilter( ignorePermissions );
 	}
 
     /**
@@ -903,7 +914,7 @@ public class DeleteNode extends DMLModStatementNode
 	  *      reference.
 	  *	6)	adds the triggers to an evolving list of triggers
 	  *
-	  *	@param	conglomVector		OUT: vector of affected indices
+      * @param  conglomerates       OUT: list of affected indices
 	  *	@param	relevantTriggers	IN/OUT. Passed in as an empty list. Filled in as we go.
 	  *	@param	needsDeferredProcessing			IN/OUT. true if the statement already needs
 	  *											deferred processing. set while evaluating this
@@ -917,8 +928,8 @@ public class DeleteNode extends DMLModStatementNode
 	private static FormatableBitSet getDeleteReadMap
 	(
 		TableDescriptor				baseTable,
-		List						conglomVector,
-		GenericDescriptorList		relevantTriggers,
+        List<ConglomerateDescriptor>  conglomerates,
+        TriggerDescriptorList       relevantTriggers,
 		boolean[]					needsDeferredProcessing
 	)
 		throws StandardException
@@ -941,7 +952,8 @@ public class DeleteNode extends DMLModStatementNode
 		** Adding indexes also takes care of the replication 
 		** requirement of having the primary key.
 		*/
-		DMLModStatementNode.getXAffectedIndexes(baseTable,  null, columnMap, conglomVector );
+        DMLModStatementNode.getXAffectedIndexes(
+                baseTable, null, columnMap, conglomerates);
 
 		/*
 	 	** If we have any DELETE triggers, then do one of the following
@@ -960,9 +972,8 @@ public class DeleteNode extends DMLModStatementNode
 			needsDeferredProcessing[0] = true;
 			
 			boolean needToIncludeAllColumns = false;
-            for (Iterator descIter = relevantTriggers.iterator();
-                    descIter.hasNext(); ) {
-                TriggerDescriptor trd = (TriggerDescriptor)descIter.next();
+
+            for (TriggerDescriptor trd : relevantTriggers) {
 				//Does this trigger have REFERENCING clause defined on it.
 				//If yes, then read all the columns from the trigger table.
 				if (!trd.getReferencingNew() && !trd.getReferencingOld())
@@ -997,19 +1008,16 @@ public class DeleteNode extends DMLModStatementNode
 		if ( correlationName == null ) { return; }
 
 		TableName	correlationNameNode = makeTableName( null, correlationName );
-		int			count = rcl.size();
 
-		for ( int i = 0; i < count; i++ )
+        for (ResultColumn column : rcl)
 		{
-			ResultColumn	column = (ResultColumn) rcl.elementAt( i );
-
 			ValueNode		expression = column.getExpression();
 
 			if ( (expression != null) && (expression instanceof ColumnReference) )
 			{
 				ColumnReference	reference = (ColumnReference) expression;
 				
-				reference.setTableNameNode( correlationNameNode );
+				reference.setQualifiedTableName( correlationNameNode );
 			}
 		}
 		
