@@ -21,49 +21,38 @@
 
 package	org.apache.derby.impl.sql.compile;
 
-import org.apache.derby.iapi.services.io.FormatableBitSet;
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
-import org.apache.derby.iapi.error.StandardException;
-import org.apache.derby.iapi.reference.Property;
-import org.apache.derby.iapi.services.property.PropertyUtil;
-
-import org.apache.derby.iapi.sql.StatementType;
-import org.apache.derby.iapi.sql.compile.CompilerContext;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-
-import org.apache.derby.iapi.types.DataTypeDescriptor;
-import org.apache.derby.iapi.types.TypeId;
-
-import org.apache.derby.catalog.types.DefaultInfoImpl;
-
-import org.apache.derby.iapi.sql.dictionary.ColumnDescriptorList;
-import org.apache.derby.iapi.sql.dictionary.ConstraintDescriptor;
-import org.apache.derby.iapi.sql.dictionary.DataDictionary;
-import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
-import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
-
-import org.apache.derby.iapi.sql.depend.DependencyManager;
-import org.apache.derby.iapi.sql.depend.ProviderInfo;
-import org.apache.derby.iapi.sql.depend.ProviderList;
-
-import org.apache.derby.iapi.reference.SQLState;
-
-import org.apache.derby.impl.sql.execute.ColumnInfo;
-import org.apache.derby.impl.sql.execute.ConstraintInfo;
-import org.apache.derby.impl.sql.execute.ConstraintConstantAction;
-import org.apache.derby.impl.sql.execute.IndexConstantAction;
-
-import	org.apache.derby.iapi.sql.dictionary.ConstraintDescriptorList;
-import org.apache.derby.iapi.sql.dictionary.ColumnDescriptor;
-
-import org.apache.derby.catalog.UUID;
-
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Properties;
 import java.util.Set;
+import org.apache.derby.catalog.UUID;
+import org.apache.derby.catalog.types.DefaultInfoImpl;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.Property;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.io.FormatableBitSet;
+import org.apache.derby.iapi.services.property.PropertyUtil;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.sql.StatementType;
+import org.apache.derby.iapi.sql.compile.CompilerContext;
+import org.apache.derby.iapi.sql.depend.DependencyManager;
+import org.apache.derby.iapi.sql.depend.ProviderInfo;
+import org.apache.derby.iapi.sql.depend.ProviderList;
+import org.apache.derby.iapi.sql.dictionary.ColumnDescriptor;
+import org.apache.derby.iapi.sql.dictionary.ColumnDescriptorList;
+import org.apache.derby.iapi.sql.dictionary.ConstraintDescriptor;
+import org.apache.derby.iapi.sql.dictionary.ConstraintDescriptorList;
+import org.apache.derby.iapi.sql.dictionary.DataDictionary;
+import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
+import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
+import org.apache.derby.iapi.types.DataTypeDescriptor;
+import org.apache.derby.iapi.types.TypeId;
+import org.apache.derby.impl.sql.execute.ColumnInfo;
+import org.apache.derby.impl.sql.execute.ConstraintConstantAction;
+import org.apache.derby.impl.sql.execute.ConstraintInfo;
+import org.apache.derby.impl.sql.execute.IndexConstantAction;
 
 /**
  * A TableElementList represents the list of columns and other table elements
@@ -71,18 +60,22 @@ import java.util.Set;
  *
  */
 
-public class TableElementList extends QueryTreeNodeVector
+class TableElementList extends QueryTreeNodeVector<TableElementNode>
 {
 	private int				numColumns;
 	private TableDescriptor td;
 
+    public TableElementList(ContextManager cm) {
+        super(TableElementNode.class, cm);
+    }
+
 	/**
 	 * Add a TableElementNode to this TableElementList
 	 *
 	 * @param tableElement	The TableElementNode to add to this list
 	 */
 
-	public void addTableElement(TableElementNode tableElement)
+    void addTableElement(TableElementNode tableElement)
 	{
 		addElement(tableElement);
 		if ((tableElement instanceof ColumnDefinitionNode) ||
@@ -100,17 +93,12 @@ public class TableElementList extends QueryTreeNodeVector
 	void setCollationTypesOnCharacterStringColumns(SchemaDescriptor sd)
         throws StandardException
     {
-		int			size = size();
-		int collationType = sd.getCollationType();
-		for (int index = 0; index < size; index++)
+        for (TableElementNode te : this)
 		{
-			TableElementNode tableElement = (TableElementNode) elementAt(index);
-
-			if (tableElement instanceof ColumnDefinitionNode)
+            if (te instanceof ColumnDefinitionNode)
 			{
-				ColumnDefinitionNode cdn = (ColumnDefinitionNode) elementAt(index);
-
-                setCollationTypeOnCharacterStringColumn( sd, cdn );
+                setCollationTypeOnCharacterStringColumn(
+                        sd, (ColumnDefinitionNode)te );
 			}
 		}
 	}
@@ -133,11 +121,7 @@ public class TableElementList extends QueryTreeNodeVector
         DataTypeDescriptor  dtd = cdn.getType();
         if ( dtd == null )
         {
-            if ( cdn.hasGenerationClause() )
-            {
-                return;
-            }
-            else
+            if ( !cdn.hasGenerationClause() )
             {
                 throw StandardException.newException
                     ( SQLState.LANG_NEEDS_DATATYPE, cdn.getColumnName() );
@@ -168,10 +152,10 @@ public class TableElementList extends QueryTreeNodeVector
 		int numAutoCols = 0;
 
 		int			size = size();
-        HashSet columnNames = new HashSet(size + 2, 0.999f);
-        HashSet constraintNames = new HashSet(size + 2, 0.999f);
+        HashSet<String> columnNames = new HashSet<String>(size + 2, 0.999f);
+        HashSet<String> constraintNames = new HashSet<String>(size + 2, 0.999f);
 		//all the primary key/unique key constraints for this table
-        ArrayList constraintsVector = new ArrayList();
+        ArrayList<Object> constraints = new ArrayList<Object>();
 
 		//special case for alter table (td is not null in case of alter table)
 		if (td != null)
@@ -191,7 +175,7 @@ public class TableElementList extends QueryTreeNodeVector
 					if (cd.getConstraintType() == DataDictionary.PRIMARYKEY_CONSTRAINT ||
 					cd.getConstraintType() == DataDictionary.UNIQUE_CONSTRAINT)
                     {
-                        constraintsVector.add(cd);
+                        constraints.add(cd);
                     }
 				}
 			}
@@ -201,13 +185,11 @@ public class TableElementList extends QueryTreeNodeVector
 		if (ddlStmt instanceof CreateTableNode)
 			tableType = ((CreateTableNode)ddlStmt).tableType;
 
-		for (int index = 0; index < size; index++)
+        for (TableElementNode tableElement : this)
 		{
-			TableElementNode tableElement = (TableElementNode) elementAt(index);
-
 			if (tableElement instanceof ColumnDefinitionNode)
 			{
-				ColumnDefinitionNode cdn = (ColumnDefinitionNode) elementAt(index);
+                ColumnDefinitionNode cdn = (ColumnDefinitionNode)tableElement;
 				if (tableType == TableDescriptor.GLOBAL_TEMPORARY_TABLE_TYPE &&
 					(cdn.getType().getTypeId().isLongConcatableTypeId() ||
 					cdn.getType().getTypeId().isUserDefinedTypeId()))
@@ -252,11 +234,11 @@ public class TableElementList extends QueryTreeNodeVector
 
 			cdn.bind(ddlStmt, dd);
 
-			//if constraint is primary key or unique key, add it to the vector
+            // If constraint is primary key or unique key, add it to the list.
 			if (cdn.getConstraintType() == DataDictionary.PRIMARYKEY_CONSTRAINT ||
 			cdn.getConstraintType() == DataDictionary.UNIQUE_CONSTRAINT)
 			{
-				/* In case of create table, the vector can have only ConstraintDefinitionNode
+                /* In case of create table, the list can have only ConstraintDefinitionNode
 				* elements. In case of alter table, it can have both ConstraintDefinitionNode
 				* (for new constraints) and ConstraintDescriptor(for pre-existing constraints).
 				*/
@@ -265,10 +247,10 @@ public class TableElementList extends QueryTreeNodeVector
 				String destName = null;
 				String[] destColumnNames = null;
 
-				for (int i=0; i<constraintsVector.size();i++)
+                for (int i = 0; i < constraints.size(); i++)
 				{
 
-					destConstraint = constraintsVector.get(i);
+                    destConstraint = constraints.get(i);
 					if (destConstraint instanceof ConstraintDefinitionNode)
 					{
 						ConstraintDefinitionNode destCDN = (ConstraintDefinitionNode)destConstraint;
@@ -287,14 +269,15 @@ public class TableElementList extends QueryTreeNodeVector
 						throw StandardException.newException(SQLState.LANG_MULTIPLE_CONSTRAINTS_WITH_SAME_COLUMNS,
 						cdn.getConstraintMoniker(), destName);
 				}
-				constraintsVector.add(cdn);
+                constraints.add(cdn);
 			}
 
 			/* Make sure that there are no duplicate constraint names in the list */
             checkForDuplicateConstraintNames(ddlStmt, constraintNames, cdn.getConstraintMoniker());
 
 			/* Make sure that the constraint we are trying to drop exists */
-			if (cdn.getConstraintType() == DataDictionary.DROP_CONSTRAINT)
+            if (cdn.getConstraintType() == DataDictionary.DROP_CONSTRAINT ||
+                cdn.getConstraintType() == DataDictionary.MODIFY_CONSTRAINT)
 			{
 				/*
 				** If no schema descriptor, then must be an invalid
@@ -316,7 +299,7 @@ public class TableElementList extends QueryTreeNodeVector
 										false);
 					if (cd == null)
 					{
-						throw StandardException.newException(SQLState.LANG_DROP_NON_EXISTENT_CONSTRAINT,
+                        throw StandardException.newException(SQLState.LANG_DROP_OR_ALTER_NON_EXISTING_CONSTRAINT,
 								(sd.getSchemaName() + "."+ dropConstraintName),
 								td.getQualifiedName());
 					}
@@ -352,8 +335,14 @@ public class TableElementList extends QueryTreeNodeVector
 		}
 
 		/* Can have only one autoincrement column in DB2 mode */
-		if (numAutoCols > 1)
+        // Raise an error if we have more than one autoincrement column in a
+        // CREATE TABLE statement, or if we are adding a new autoincrement
+        // column to an existing table that already has an autoincrement column,
+        // with ALTER TABLE ADD COLUMN.
+        if (numAutoCols > 1 ||
+                (numAutoCols > 0 && td != null && td.tableHasAutoincrement())) {
 			throw StandardException.newException(SQLState.LANG_MULTIPLE_AUTOINCREMENT_COLUMNS);
+        }
 
 	}
 
@@ -367,11 +356,8 @@ public class TableElementList extends QueryTreeNodeVector
     public  void    validatePrimaryKeyNullability()
         throws StandardException
     {
-		int			size = size();
-		for (int index = 0; index < size; index++)
+        for (TableElementNode tableElement : this)
 		{
-			TableElementNode tableElement = (TableElementNode) elementAt(index);
-
 			if (! (tableElement.hasConstraint()))
 			{
 				continue;
@@ -402,27 +388,17 @@ public class TableElementList extends QueryTreeNodeVector
 	 *
 	 * @return int	The number of constraints of the specified type.
 	 */
-	public int countConstraints(int constraintType)
+    int countConstraints(int constraintType)
 	{
 		int	numConstraints = 0;
-		int size = size();
 
-		for (int index = 0; index < size; index++)
+        for (TableElementNode element : this)
 		{
-			ConstraintDefinitionNode cdn;
-			TableElementNode element = (TableElementNode) elementAt(index);
-
-			if (! (element instanceof ConstraintDefinitionNode))
-			{
-				continue;
-			}
-
-			cdn = (ConstraintDefinitionNode) element;
-
-			if (constraintType == cdn.getConstraintType())
-			{
-				numConstraints++;
-			}
+            if (element instanceof ConstraintDefinitionNode &&
+                ((ConstraintDefinitionNode)element).getConstraintType() ==
+                    constraintType) {
+                numConstraints++;
+            }
 		}
 
 		return numConstraints;
@@ -431,27 +407,16 @@ public class TableElementList extends QueryTreeNodeVector
     /**
 	 * Count the number of generation clauses.
 	 */
-	public int countGenerationClauses()
+    int countGenerationClauses()
 	{
 		int	numGenerationClauses = 0;
-		int size = size();
 
-		for (int index = 0; index < size; index++)
+        for (TableElementNode element : this)
 		{
-			ColumnDefinitionNode cdn;
-			TableElementNode element = (TableElementNode) elementAt(index);
-
-			if (! (element instanceof ColumnDefinitionNode))
-			{
-				continue;
-			}
-
-			cdn = (ColumnDefinitionNode) element;
-
-			if ( cdn.hasGenerationClause() )
-			{
-				numGenerationClauses++;
-			}
+            if (element instanceof ColumnDefinitionNode &&
+                    ((ColumnDefinitionNode)element).hasGenerationClause()) {
+                numGenerationClauses++;
+            }
 		}
 
 		return numGenerationClauses;
@@ -462,7 +427,7 @@ public class TableElementList extends QueryTreeNodeVector
 	 *
 	 * @return int	The number of columns.
 	 */
-	public int countNumberOfColumns()
+    int countNumberOfColumns()
 	{
 		return numColumns;
 	}
@@ -474,7 +439,7 @@ public class TableElementList extends QueryTreeNodeVector
 	 *
 	 * @return int		The number of constraints in the create table.
 	 */
-	public int genColumnInfos( ColumnInfo[] colInfos)
+    int genColumnInfos( ColumnInfo[] colInfos)
         throws StandardException
 	{
 		int	numConstraints = 0;
@@ -482,15 +447,15 @@ public class TableElementList extends QueryTreeNodeVector
 
 		for (int index = 0; index < size; index++)
 		{
-			if (((TableElementNode) elementAt(index)).getElementType() == TableElementNode.AT_DROP_COLUMN)
+            if (elementAt(index).getElementType() == TableElementNode.AT_DROP_COLUMN)
 			{
-                String columnName = ((TableElementNode) elementAt(index)).getName();
+                String columnName = elementAt(index).getName();
 
 				colInfos[index] = new ColumnInfo(
 								columnName,
 								td.getColumnDescriptor( columnName ).getType(),
                                 null, null, null, null, null,
-								ColumnInfo.DROP, 0, 0, 0);
+								ColumnInfo.DROP, 0, 0, 0, 0);
 				break;
 			}
 
@@ -539,11 +504,13 @@ public class TableElementList extends QueryTreeNodeVector
 								coldef.getAutoincrementStart() : 0),
 							   (coldef.isAutoincrementColumn() ? 
 								coldef.getAutoincrementIncrement() : 0),
+							   (coldef.isAutoincrementColumn() ?
+ 								coldef.getAutoincrementCycle() : 0),
 							   (coldef.isAutoincrementColumn() ? 
 								coldef.getAutoinc_create_or_modify_Start_Increment() : -1));
 
 			/* Remember how many constraints that we've seen */
-			if (coldef.hasConstraint())
+        if (coldef.hasConstraint())
 			{
 				numConstraints++;
 			}
@@ -559,7 +526,7 @@ public class TableElementList extends QueryTreeNodeVector
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void appendNewColumnsToRCL(FromBaseTable table)
+    void appendNewColumnsToRCL(FromBaseTable table)
 		throws StandardException
 	{
 		int				 size = size();
@@ -575,18 +542,13 @@ public class TableElementList extends QueryTreeNodeVector
 				ValueNode		valueNode;
 
 				/* Build a ResultColumn/BaseColumnNode pair for the column */
-				valueNode = (ValueNode) getNodeFactory().getNode(
-											C_NodeTypes.BASE_COLUMN_NODE,
-											cdn.getColumnName(),
+                valueNode = new BaseColumnNode(cdn.getColumnName(),
 									  		exposedName,
 											cdn.getType(),
 											getContextManager());
 
-				resultColumn = (ResultColumn) getNodeFactory().getNode(
-												C_NodeTypes.RESULT_COLUMN,
-												cdn.getType(), 
-												valueNode,
-												getContextManager());
+                resultColumn = new ResultColumn(
+                        cdn.getType(), valueNode, getContextManager());
 				resultColumn.setName(cdn.getColumnName());
 				rcl.addElement(resultColumn);
 			}
@@ -604,18 +566,14 @@ public class TableElementList extends QueryTreeNodeVector
 	void bindAndValidateCheckConstraints(FromList fromList)
 		throws StandardException
 	{
-		CompilerContext cc;
-		FromBaseTable				table = (FromBaseTable) fromList.elementAt(0);
-		int						  size = size();
+        FromBaseTable table = (FromBaseTable) fromList.elementAt(0);
+        CompilerContext cc = getCompilerContext();
 
-		cc = getCompilerContext();
+        ArrayList<AggregateNode> aggregates = new ArrayList<AggregateNode>();
 
-		ArrayList aggregateVector = new ArrayList();
-
-		for (int index = 0; index < size; index++)
+        for (TableElementNode element : this)
 		{
 			ConstraintDefinitionNode cdn;
-			TableElementNode element = (TableElementNode) elementAt(index);
 			ValueNode	checkTree;
 
 			if (! (element instanceof ConstraintDefinitionNode))
@@ -652,11 +610,11 @@ public class TableElementList extends QueryTreeNodeVector
 
 				// Tell the compiler context to only allow deterministic nodes
 				cc.setReliability( CompilerContext.CHECK_CONSTRAINT );
-				checkTree = checkTree.bindExpression(fromList, (SubqueryList) null,
-										 aggregateVector);
+                checkTree = checkTree.bindExpression(
+                        fromList, (SubqueryList) null, aggregates);
 
 				// no aggregates, please
-				if (!aggregateVector.isEmpty())
+                if (!aggregates.isEmpty())
 				{
 					throw StandardException.newException(SQLState.LANG_INVALID_CHECK_CONSTRAINT, cdn.getConstraintText());
 				}
@@ -678,24 +636,14 @@ public class TableElementList extends QueryTreeNodeVector
 				cc.setReliability(previousReliability);
 			}
 	
-			/* We have a valid check constraint, now build an array of
-			 * 1-based columnIds that the constraint references.
-			 */
-			ResultColumnList rcl = table.getResultColumns();
-			int		numReferenced = rcl.countReferencedColumns();
-			int[]	checkColumnReferences = new int[numReferenced];
-
-			rcl.recordColumnReferences(checkColumnReferences, 1);
-			cdn.setCheckColumnReferences(checkColumnReferences);
-
-			/* Now we build a list with only the referenced columns and
+            /* We have a valid check constraint.
+             * Now we build a list with only the referenced columns and
 			 * copy it to the cdn.  Thus we can build the array of
 			 * column names for the referenced columns during generate().
 			 */
-			ResultColumnList refRCL =
-						(ResultColumnList) getNodeFactory().getNode(
-												C_NodeTypes.RESULT_COLUMN_LIST,
-												getContextManager());
+            ResultColumnList rcl = table.getResultColumns();
+            int numReferenced = rcl.countReferencedColumns();
+            ResultColumnList refRCL = new ResultColumnList(getContextManager());
 			rcl.copyReferencedColumnsToNewList(refRCL);
 
 			/* A column check constraint can only refer to that column. If this is a
@@ -703,9 +651,9 @@ public class TableElementList extends QueryTreeNodeVector
 			 */
 			if (cdn.getColumnList() != null)
 			{
-				String colName = ((ResultColumn)(cdn.getColumnList().elementAt(0))).getName();
+                String colName = cdn.getColumnList().elementAt(0).getName();
 				if (numReferenced > 1 ||
-					!colName.equals(((ResultColumn)(refRCL.elementAt(0))).getName()))
+                    !colName.equals(refRCL.elementAt(0).getName()))
 					throw StandardException.newException(SQLState.LANG_DB2_INVALID_CHECK_CONSTRAINT, colName);
 				
 			}
@@ -715,6 +663,9 @@ public class TableElementList extends QueryTreeNodeVector
 			 * starts with a clean list.
 			 */
 			rcl.clearColumnReferences();
+
+            // Make sure all names are schema qualified (DERBY-6362)
+            cdn.qualifyNames();
 		}
 	}
 
@@ -732,25 +683,22 @@ public class TableElementList extends QueryTreeNodeVector
 	void bindAndValidateGenerationClauses( SchemaDescriptor sd, FromList fromList, FormatableBitSet generatedColumns, TableDescriptor baseTable )
 		throws StandardException
 	{
-		CompilerContext cc;
-		FromBaseTable				table = (FromBaseTable) fromList.elementAt(0);
-        ResultColumnList            tableColumns = table.getResultColumns();
-        int                                 columnCount = table.getResultColumns().size();
-		int						  size = size();
+        FromBaseTable    table = (FromBaseTable) fromList.elementAt(0);
+        ResultColumnList tableColumns = table.getResultColumns();
+        int              columnCount = table.getResultColumns().size();
 
         // complain if a generation clause references another generated column
         findIllegalGenerationReferences( fromList, baseTable );
 
         generatedColumns.grow( columnCount + 1 );
         
-		cc = getCompilerContext();
+        CompilerContext cc = getCompilerContext();
 
-		ArrayList aggregateVector = new ArrayList();
+        ArrayList<AggregateNode> aggregates = new ArrayList<AggregateNode>();
 
-		for (int index = 0; index < size; index++)
+        for (TableElementNode element : this)
 		{
 			ColumnDefinitionNode cdn;
-			TableElementNode element = (TableElementNode) elementAt(index);
             GenerationClauseNode    generationClauseNode;
 			ValueNode	generationTree;
 
@@ -788,8 +736,10 @@ public class TableElementList extends QueryTreeNodeVector
 				// Tell the compiler context to forbid subqueries and
 				// non-deterministic functions.
 				cc.setReliability( CompilerContext.GENERATION_CLAUSE_RESTRICTION );
-				generationTree = generationClauseNode.bindExpression(fromList, (SubqueryList) null,
-										 aggregateVector);
+                generationTree = generationClauseNode.bindExpression(
+                        fromList, (SubqueryList) null, aggregates);
+
+                SelectNode.checkNoWindowFunctions(generationClauseNode, "generation clause");
 
                 //
                 // If the user did not declare a type for this column, then the column type defaults
@@ -831,7 +781,7 @@ public class TableElementList extends QueryTreeNodeVector
                 }
 
 				// no aggregates, please
-				if (!aggregateVector.isEmpty())
+                if (!aggregates.isEmpty())
 				{
 					throw StandardException.newException( SQLState.LANG_AGGREGATE_IN_GENERATION_CLAUSE, cdn.getName());
 				}
@@ -866,7 +816,9 @@ public class TableElementList extends QueryTreeNodeVector
 
             for ( int i = 0; i < numReferenced; i++ )
             {
-                referencedColumnNames[ i ] = ((ResultColumn)rcl.elementAt( generationClauseColumnReferences[ i ] - 1 )).getName();
+                referencedColumnNames[i] =
+                    rcl.elementAt(generationClauseColumnReferences[i] - 1).
+                        getName();
             }
 
             String              currentSchemaName = getLanguageConnectionContext().getCurrentSchemaName();
@@ -894,9 +846,8 @@ public class TableElementList extends QueryTreeNodeVector
 	void findIllegalGenerationReferences( FromList fromList, TableDescriptor baseTable )
 		throws StandardException
 	{
-        ArrayList   generatedColumns = new ArrayList();
-        HashSet     names = new HashSet();
-		int         size = size();
+        ArrayList<ColumnDefinitionNode>   generatedColumns = new ArrayList<ColumnDefinitionNode>();
+        HashSet<String>     names = new HashSet<String>();
 
         // add in existing generated columns if this is an ALTER TABLE statement
         if ( baseTable != null )
@@ -910,10 +861,9 @@ public class TableElementList extends QueryTreeNodeVector
         }
         
         // find all of the generated columns
-		for (int index = 0; index < size; index++)
+        for (TableElementNode element : this)
 		{
 			ColumnDefinitionNode cdn;
-			TableElementNode     element = (TableElementNode) elementAt(index);
 
 			if (! (element instanceof ColumnDefinitionNode)) { continue; }
 
@@ -930,13 +880,14 @@ public class TableElementList extends QueryTreeNodeVector
         int    count = generatedColumns.size();
         for ( int i = 0; i < count; i++ )
         {
-            ColumnDefinitionNode    cdn = (ColumnDefinitionNode) generatedColumns.get( i );
+            ColumnDefinitionNode    cdn = generatedColumns.get( i );
             GenerationClauseNode    generationClauseNode = cdn.getGenerationClauseNode();
-            List                    referencedColumns = generationClauseNode.findReferencedColumns();
+            List<ColumnReference>   referencedColumns =
+                generationClauseNode.findReferencedColumns();
             int                     refCount = referencedColumns.size();
             for ( int j = 0; j < refCount; j++ )
             {
-                String name = ((ColumnReference) referencedColumns.get(j)).getColumnName();
+                String name = referencedColumns.get(j).getColumnName();
 
                 if ( name != null )
                 {
@@ -969,14 +920,11 @@ public class TableElementList extends QueryTreeNodeVector
         
 		FromBaseTable				table = (FromBaseTable) fromList.elementAt(0);
         ResultColumnList        tableColumns = table.getResultColumns();
-		int						  size = size();
 
         // loop through the foreign keys, looking for keys which violate the
         // rulse we're enforcing
-		for (int index = 0; index < size; index++)
+        for (TableElementNode element : this)
 		{
-			TableElementNode element = (TableElementNode) elementAt(index);
-
 			if (! (element instanceof FKConstraintDefinitionNode))
 			{
 				continue;
@@ -1011,14 +959,10 @@ public class TableElementList extends QueryTreeNodeVector
             // SET DEFAULT or whose update rule is ON UPDATE CASCADE.
             // See if any of the key columns are generated columns.
             //
-            ResultColumnList                keyCols = fk.getColumnList();
-            int                                     keyCount = keyCols.size();
-
-            for ( int i = 0; i < keyCount; i++ )
+            for (ResultColumn keyCol : fk.getColumnList())
             {
-                ResultColumn    keyCol = (ResultColumn) keyCols.elementAt( i );
-                String                  keyColName = keyCol.getName();
-                int     position = tableColumns.getPosition( keyColName, 1 );
+                String keyColName = keyCol.getName();
+                int position = tableColumns.getPosition( keyColName, 1 );
 
                 if ( generatedColumns.isSet(  position ) )
                 {
@@ -1047,20 +991,15 @@ public class TableElementList extends QueryTreeNodeVector
 				DataDictionary dd)
 		throws StandardException
 	{
-		int size = size();
 		int conActionIndex = 0;
-		for (int index = 0; index < size; index++)
+
+        for (TableElementNode ten : this)
 		{
 			String[]	columnNames = null;
-			TableElementNode ten = (TableElementNode) elementAt(index);
 			IndexConstantAction indexAction = null;
 
-			if (! ten.hasConstraint())
-			{
-				continue;
-			}
-
-			if (ten instanceof ColumnDefinitionNode)
+            if (! ten.hasConstraint() ||
+                ten instanceof ColumnDefinitionNode)
 			{
 				continue;
 			}
@@ -1074,6 +1013,7 @@ public class TableElementList extends QueryTreeNodeVector
 			}
 
 			int constraintType = constraintDN.getConstraintType();
+            boolean[] cChars = constraintDN.getCharacteristics();
 			String constraintText = constraintDN.getConstraintText();
 
 			/*
@@ -1117,15 +1057,22 @@ public class TableElementList extends QueryTreeNodeVector
 						forCreateTable,
 						unique,
                         uniqueWithDuplicateNulls,
+                        cChars[0], // deferrable?
+                        cChars[1], // initiallyDeferred?
 						null, constraintDN,
 						columnNames, true, tableSd, tableName,
 						constraintType, dd);
 				} 
                 else 
-                {
+                {   // PRIMARY KEY, FOREIGN KEY
+                    // For foreign key constraint we do no mark the
+                    // index as deferrable; since checking isn't done on
+                    // duplicate keys there.
 					indexAction = genIndexAction(
 						forCreateTable,
 						constraintDN.requiresUniqueIndex(), false,
+                        cChars[0],
+                        cChars[1],
 						null, constraintDN,
 						columnNames, true, tableSd, tableName,
 						constraintType, dd);
@@ -1151,11 +1098,24 @@ public class TableElementList extends QueryTreeNodeVector
 												 constraintDN.getDropBehavior(),
                                                  constraintDN.getVerifyType());
 			}
+            else if (constraintType == DataDictionary.MODIFY_CONSTRAINT) {
+                conActions[conActionIndex] =
+                    getGenericConstantActionFactory().
+                        getAlterConstraintConstantAction(
+                                                 constraintName,
+                                                 constraintDN.getDropSchemaName(),
+                                                 cChars,
+                                                 tableName,
+                                                 td.getUUID(),
+                                                 tableSd.getSchemaName(),
+                                                 indexAction);
+
+            }
 			else
 			{
 				ProviderList apl = constraintDN.getAuxiliaryProviderList();
 				ConstraintInfo refInfo = null;
-				ProviderInfo[]	providerInfos = null;
+                ProviderInfo[]  providerInfos;
 
 				if (constraintDN instanceof FKConstraintDefinitionNode)
 				{
@@ -1182,6 +1142,7 @@ public class TableElementList extends QueryTreeNodeVector
 						getCreateConstraintConstantAction(
 												 constraintName, 
 											     constraintType,
+                                                 cChars,
                                                  forCreateTable,
 												 tableName, 
 												 ((td != null) ? td.getUUID() : (UUID) null),
@@ -1189,7 +1150,6 @@ public class TableElementList extends QueryTreeNodeVector
 												 columnNames,
 												 indexAction,
 												 constraintText,
-												 true, 		// enabled
 												 refInfo,
 												 providerInfos);
 			}
@@ -1200,7 +1160,6 @@ public class TableElementList extends QueryTreeNodeVector
 	private boolean columnsMatch(String[] columnNames1, String[] columnNames2)
 	{
 		int srcCount, srcSize, destCount,destSize;
-		boolean match = true;
 
 		if (columnNames1.length != columnNames2.length)
 			return false;
@@ -1210,7 +1169,7 @@ public class TableElementList extends QueryTreeNodeVector
 
 		for (srcCount = 0; srcCount < srcSize; srcCount++)
 		{
-			match = false;
+            boolean match = false;
 			for (destCount = 0; destCount < destSize; destCount++) {
 				if (columnNames1[srcCount].equals(columnNames2[destCount])) {
 					match = true;
@@ -1237,6 +1196,10 @@ public class TableElementList extends QueryTreeNodeVector
      *                                      column in the key has a null value,
      *                                      no checking is done and insert will
      *                                      always succeed.
+     * @param hasDeferrableChecking         True if index is used to back a
+     *                                      deferrable constraint
+     * @param initiallyDeferred             True means the deferrable constraint
+     *                                      has deferred mode
      * @param indexName	                    The type of index (BTREE, for 
      *                                      example)
      * @param cdn
@@ -1253,6 +1216,8 @@ public class TableElementList extends QueryTreeNodeVector
     boolean                     forCreateTable,
     boolean                     isUnique,
     boolean                     isUniqueWithDuplicateNulls,
+    boolean                     hasDeferrableChecking,
+    boolean                     initiallyDeferred,
     String                      indexName,
     ConstraintDefinitionNode    cdn,
     String[]                    columnNames,
@@ -1296,6 +1261,9 @@ public class TableElementList extends QueryTreeNodeVector
                     forCreateTable, 
                     isUnique, 
                     isUniqueWithDuplicateNulls,
+                    hasDeferrableChecking,
+                    initiallyDeferred,
+                    constraintType,
                     "BTREE", // indexType
                     sd.getSchemaName(),
                     indexName,
@@ -1338,11 +1306,12 @@ public class TableElementList extends QueryTreeNodeVector
             // is set for the whole database or just set on this statement.
             return result;
         }
-        ResultColumnList rcl = cdn.getColumnList();
+
         int approxLength = 0;
-        for (int index = 0; index < rcl.size(); index++)
+
+        for (ResultColumn rc : cdn.getColumnList())
         {
-            String colName = ((ResultColumn) rcl.elementAt(index)).getName();
+            String colName = rc.getName();
             DataTypeDescriptor dtd;
             if (td == null)
                 dtd = getColumnDataTypeDescriptor(colName);
@@ -1378,7 +1347,7 @@ public class TableElementList extends QueryTreeNodeVector
 	 * @exception StandardException		Thrown on error
 	 */
 	private void checkForDuplicateColumns(DDLStatementNode ddlStmt,
-									Set seenNames,
+									Set<String> seenNames,
 									String colName)
 			throws StandardException
 	{
@@ -1406,7 +1375,7 @@ public class TableElementList extends QueryTreeNodeVector
 	 * @exception StandardException		Thrown on error
 	 */
 	private void checkForDuplicateConstraintNames(DDLStatementNode ddlStmt,
-									Set seenNames,
+									Set<String> seenNames,
 									String constraintName)
 			throws StandardException
 	{
@@ -1471,13 +1440,9 @@ public class TableElementList extends QueryTreeNodeVector
 	 */
 	private void setColumnListToNotNull(ConstraintDefinitionNode cdn)
 	{
-		ResultColumnList rcl = cdn.getColumnList();
-		int rclSize = rcl.size();
-		for (int index = 0; index < rclSize; index++)
+        for (ResultColumn rc : cdn.getColumnList())
 		{
-			String colName = ((ResultColumn) rcl.elementAt(index)).getName();
-            
-            findColumnDefinition(colName).setNullability(false);
+            findColumnDefinition(rc.getName()).setNullability(false);
         }
 	}
 
@@ -1493,20 +1458,14 @@ public class TableElementList extends QueryTreeNodeVector
     ConstraintDefinitionNode    cdn, 
     TableDescriptor             td) 
     {
-        ResultColumnList rcl = cdn.getColumnList();
-        int rclSize = rcl.size();
-        for (int index = 0; index < rclSize; index++)
+        for (ResultColumn rc : cdn.getColumnList())
         {
-            String colName = ((ResultColumn) rcl.elementAt(index)).getName();
-            DataTypeDescriptor dtd;
-            if (td == null)
-            {
-                dtd = getColumnDataTypeDescriptor(colName);
-            }
-            else
-            {
-                dtd = getColumnDataTypeDescriptor(colName, td);
-            }
+            String colName = rc.getName();
+
+            DataTypeDescriptor dtd = (td == null) ?
+                getColumnDataTypeDescriptor(colName) :
+                getColumnDataTypeDescriptor(colName, td);
+
             // todo dtd may be null if the column does not exist, we should check that first
             if (dtd != null && dtd.isNullable())
             {
@@ -1518,20 +1477,12 @@ public class TableElementList extends QueryTreeNodeVector
 
     private void checkForNullColumns(ConstraintDefinitionNode cdn, TableDescriptor td) throws StandardException
     {
-        ResultColumnList rcl = cdn.getColumnList();
-        int rclSize = rcl.size();
-        for (int index = 0; index < rclSize; index++)
+        for (ResultColumn rc : cdn.getColumnList())
         {
-            String colName = ((ResultColumn) rcl.elementAt(index)).getName();
-            DataTypeDescriptor dtd;
-            if (td == null)
-            {
-                dtd = getColumnDataTypeDescriptor(colName);
-            }
-            else
-            {
-                dtd = getColumnDataTypeDescriptor(colName, td);
-            }
+            DataTypeDescriptor dtd = (td == null) ?
+                    getColumnDataTypeDescriptor(rc.getName()) :
+                    getColumnDataTypeDescriptor(rc.getName(), td);
+
             // todo dtd may be null if the column does not exist, we should check that first
             if (dtd != null && dtd.isNullable())
             {
@@ -1541,7 +1492,7 @@ public class TableElementList extends QueryTreeNodeVector
                     ? SQLState.LANG_ADD_PRIMARY_KEY_ON_NULL_COLS
                     : SQLState.LANG_DB2_ADD_UNIQUE_OR_PRIMARY_KEY_ON_NULL_COLS;
 
-                throw StandardException.newException(errorState, colName);
+                throw StandardException.newException(errorState, rc.getName());
             }
         }
     }
@@ -1575,12 +1526,9 @@ public class TableElementList extends QueryTreeNodeVector
      * not in the list.
      */
     private ColumnDefinitionNode findColumnDefinition(String colName) {
-        int size = size();
-        for (int index = 0; index < size; index++) {
-            TableElementNode tableElement = (TableElementNode) elementAt(index);
-
-            if (tableElement instanceof ColumnDefinitionNode) {
-                ColumnDefinitionNode cdn = (ColumnDefinitionNode) tableElement;
+        for (TableElementNode te : this) {
+            if (te instanceof ColumnDefinitionNode) {
+                ColumnDefinitionNode cdn = (ColumnDefinitionNode) te;
                 if (colName.equals(cdn.getName())) {
                     return cdn;
                 }
@@ -1599,7 +1547,7 @@ public class TableElementList extends QueryTreeNodeVector
      * 
      * @return boolean Whether or not a match is found.
      */
-	public boolean containsColumnName(String colName)
+    boolean containsColumnName(String colName)
 	{
         return findColumnDefinition(colName) != null;
 	}
