@@ -25,17 +25,18 @@ package org.apache.derbyTesting.functionTests.harness;
  * Sed.java
  *
  * This is a version of "sed" in Java for the Derby Function Tests,
- * written using the OROMatcher Perl5 regular expression classes.
+ * written using the java.util.regex regular expression classes.
  * The substitutions/deletions are based on the original kornshell tests.
  *
  ***/
 
 import java.io.*;
 import java.util.Vector;
-import org.apache.oro.text.regex.*;
-import java.util.Enumeration;
+import java.util.List;
 import java.util.Properties;
 import java.util.StringTokenizer;
+import java.util.regex.Pattern;
+import java.util.stream.Collectors;
 import org.apache.derbyTesting.functionTests.util.TestUtil;
 
 public class Sed
@@ -65,7 +66,7 @@ public class Sed
     	String hostName = TestUtil.getHostName();
     	
         // Vector for storing lines to be deleted
-        Vector deleteLines = new Vector();
+        Vector<String> deleteLines = new Vector<String>();
         deleteLines.addElement("^ij version.*$");
         deleteLines.addElement("^\\*\\*\\*\\* Test Run Started .* \\*\\*\\*\\*$");
         deleteLines.addElement("^\\*\\*\\*\\* Test Run Completed .* \\*\\*\\*\\*$");
@@ -86,7 +87,7 @@ public class Sed
         deleteLines.addElement("^.*at java.*\\<init\\>\\(.*\\(Compiled Code\\)\\)$");
         deleteLines.addElement("^(.*at .*)\\(Interpreted Code\\)$");
         deleteLines.addElement("^(.*at .*)\\(Unknown Source\\)$");
-        deleteLines.addElement("^(.*at .*)\\(Native Method\\)$");
+        deleteLines.addElement("^(.*at .*)\\(.*Native Method\\)$");
         deleteLines.addElement("^\\tat $"); // rare case of incomplete stack trace line
         deleteLines.addElement("optimizer estimated cost");
         deleteLines.addElement("optimizer estimated row count");
@@ -113,7 +114,7 @@ public class Sed
         deleteLines.addElement("^OK \\(.*$");
 
         // Vectors for substitutions
-        Vector searchStrings = new Vector();
+        Vector<String> searchStrings = new Vector<String>();
         searchStrings.addElement("^Transaction:\\(.*\\) *\\|"); 
         searchStrings.addElement("^Read [0-9]* of [0-9]* bytes$");
         searchStrings.addElement("Directory .*connect.wombat.seg0");
@@ -148,12 +149,12 @@ public class Sed
         timestampFilter.append( "[0-9][0-9]:[0-9][0-9]:[0-9][0-9].[0-9]* *" );
         searchStrings.addElement( timestampFilter.toString() );
         // Filter remove transaction id's from deadlock messages
-        searchStrings.addElement("^  Waiting XID : {.*}");
+        searchStrings.addElement("^  Waiting XID : \\{.*\\}");
         searchStrings.addElement("^  Granted XID : .*$");
         searchStrings.addElement("^The selected victim is XID : .*");
         // Filters for build numbers
         searchStrings.addElement("(beta - )\\(([0-9]*)\\)");
-        searchStrings.addElement("Level2CostEstimateImpl: .*");
+        searchStrings.addElement("CostEstimateImpl: .*");
         // Filter for xa tests for the numbers representing the db name (it can change)
         searchStrings.addElement("^Transaction ([0-9])* : \\(([0-9]*)\\,([0-9a-f]*)\\,([0-9a-f]*)\\)");
         // Filter for optimizer number for zindexesLevel1 test (due to a change in display width for the test)
@@ -208,7 +209,7 @@ public class Sed
 			searchStrings.addElement("\\[\\.fffffffff\\]");			
 		}
 		
-        Vector subStrings = new Vector();
+        Vector<String> subStrings = new Vector<String>();
         subStrings.addElement("Transaction:(XXX)|");
         subStrings.addElement("Read ... bytes");
         subStrings.addElement("Directory DBLOCATION/seg0");
@@ -224,7 +225,7 @@ public class Sed
         subStrings.addElement("The selected victim is XID : VVV");
         // sub build numbers
         subStrings.addElement("$1(xxXXxxFILTERED-BUILD-NUMBERxxXXxx)");
-        subStrings.addElement("Level2CostEstimateImpl: xxXXxxFILTERED-INFORMATIONxxXXxx");
+        subStrings.addElement("CostEstimateImpl: xxXXxxFILTERED-INFORMATIONxxXXxx");
         // sub for db name in xa tests (it can change)
         subStrings.addElement("Transaction $1 : ($2,FILTERED,FILTERED)");
         // sub for optimizer number for zindexesLevel1 test
@@ -282,14 +283,14 @@ public class Sed
         throws IOException
     {
         // Vector for storing lines to be deleted
-        Vector deleteLines = new Vector();
+        Vector<String> deleteLines = new Vector<String>();
 
         // Vectors for substitutions
-        Vector searchStrings = new Vector();
+        Vector<String> searchStrings = new Vector<String>();
         searchStrings.addElement("[ ]*\\|");
         searchStrings.addElement("^--*");
 
-        Vector subStrings = new Vector();
+        Vector<String> subStrings = new Vector<String>();
         // true and false show up as 1 and 0 in JCC. 
         //because they have no boolean support
         subStrings.addElement(" |");
@@ -299,29 +300,22 @@ public class Sed
 
     }
 
-    private void doWork(File srcFile, File dstFile, InputStream is, Vector deleteLines, 
-        Vector searchStrings, Vector subStrings, InputStream isSed)
+    private void doWork(File srcFile, File dstFile, InputStream is,
+        Vector<String> deleteLines,  Vector<String> searchStrings,
+        Vector<String> subStrings, InputStream isSed)
         throws IOException
     {
         doWork(srcFile, dstFile, is, deleteLines, searchStrings, subStrings, isSed, false);
     }
 		
 
-    private void doWork(File srcFile, File dstFile, InputStream is, Vector deleteLines, 
-        Vector searchStrings, Vector subStrings, InputStream isSed, boolean isI18N)
+    private void doWork(File srcFile, File dstFile, InputStream is,
+        Vector<String> deleteLines, Vector<String> searchStrings,
+        Vector<String> subStrings, InputStream isSed, boolean isI18N)
         throws IOException
     {
-		
-        boolean lineDeleted = false;
-        PatternMatcher matcher;
-        Perl5Compiler pcompiler;
-        PatternMatcherInput input;
         BufferedReader inFile;
         PrintWriter outFile;
-        String result = "";
-        String regex;
-        Vector delPatternVector = new Vector();
-        Vector subPatternVector = new Vector();
 
         // ---------------------------------
         // Try loading the sed properties if they exist (see jdbc_sed.properties as an example)
@@ -330,9 +324,8 @@ public class Sed
             Properties sedp = new Properties();
 
             sedp.load(isSed);
-            for (Enumeration e = sedp.propertyNames(); e.hasMoreElements(); )
+            for (String key : sedp.stringPropertyNames())
             {
-                String key = (String)e.nextElement();
                 if (key.equals("substitute"))
                 {
                     String value = sedp.getProperty(key);
@@ -372,10 +365,6 @@ public class Sed
         }
         // ---------------------------------
 
-        //Create Perl5Compiler and Perl5Matcher
-        pcompiler = new Perl5Compiler();
-        matcher = new Perl5Matcher();
-
         // Define the input and output files based on args
         if (is == null && isI18N) {
             // read UTF-8 encoded file
@@ -391,42 +380,12 @@ public class Sed
         ( new BufferedWriter(new FileWriter(dstFile), 10000), true );
 
         // Attempt to compile the patterns for deletes
-        for (int i = 0; i < deleteLines.size(); i++)
-        {
-            try
-            {
-                regex = (String)deleteLines.elementAt(i);
-                //System.out.println("The pattern: " + regex);
-                Pattern pattern = pcompiler.compile(regex);
-                if (pattern == null)
-                    System.out.println("pattern is null");
-                delPatternVector.addElement(pattern);
-            }
-            catch(MalformedPatternException e)
-            {
-                System.out.println("Bad pattern.");
-                System.out.println(e.getMessage());
-            }
-        }
+        List<Pattern> deletePatterns = deleteLines.stream()
+                .map(Pattern::compile).collect(Collectors.toList());
 
         // Attempt to compile the patterns for substitutes
-        for (int i = 0; i < searchStrings.size(); i++)
-        {
-            try
-            {
-                regex = (String)searchStrings.elementAt(i);
-                //System.out.println("The pattern: " + regex);
-                Pattern pattern = pcompiler.compile(regex);
-                if (pattern == null)
-                    System.out.println("pattern is null");
-                subPatternVector.addElement(pattern);
-            }
-            catch(MalformedPatternException e)
-            {
-                System.out.println("Bad pattern.");
-                System.out.println(e.getMessage());
-            }
-        }
+        List<Pattern> substitutePatterns = searchStrings.stream()
+                .map(Pattern::compile).collect(Collectors.toList());
 
         String str;
         int j;
@@ -438,7 +397,7 @@ public class Sed
         
             //System.out.println("***Line no: " + lineCount);
             //System.out.println("***Line is: " + str);
-            lineDeleted = false;
+            boolean lineDeleted = false;
 
             // First delete any nulls (Cafe 1.8 leaves nulls)
             if (str.length() == 1)
@@ -493,68 +452,24 @@ public class Sed
             // Determine if this line should be deleted for delete pattern match
             if ( lineDeleted == false )
             {
-                for (j = 0; j < delPatternVector.size(); j++)
-                {
-                    if ( matcher.contains( str, (Pattern)delPatternVector.elementAt(j) ) )
-                    {
-                        //System.out.println("***Match found to delete line***");
-                        String tmpp = ((Pattern)delPatternVector.elementAt(j)).getPattern();
-                        //System.out.println("***Pattern is: " + tmpp);
-
-                        // In this case we are removing the line, so don't write it out
-                        lineDeleted = true;
-                        break;
-                    }
-                }
+                final String s = str;
+                lineDeleted =
+                    deletePatterns.stream().anyMatch(p -> p.matcher(s).find());
             }
 
             // Determine if any substitutions are needed
             if (lineDeleted == false)
             {
-                Substitution substitution;
-                StringSubstitution strsub = new StringSubstitution("");
-                Perl5Substitution perlsub = new Perl5Substitution("");
-                boolean subDone = false;
-                for (j = 0; j < subPatternVector.size(); j++)
+                for (j = 0; j < substitutePatterns.size(); j++)
                 {
-                    input = new PatternMatcherInput(str);
-                    Pattern patt = (Pattern)subPatternVector.elementAt(j);
-                    String pstr = patt.getPattern();
-                    //System.out.println("Pattern string is " + pstr);
-                    String sub = (String)subStrings.elementAt(j);
-                    if (sub.indexOf("$") > 0)
-                    {
-                        perlsub.setSubstitution(sub);
-                        substitution = (Substitution)perlsub;
-                    } else {
-                        strsub.setSubstitution(sub);
-                        substitution = (Substitution)strsub;
-                    }
+                    Pattern patt = substitutePatterns.get(j);
+                    //System.out.println("Pattern string is " + patt);
+                    String sub = subStrings.get(j);
                     //System.out.println("Substitute str = " + sub);
-                    if ( matcher.contains( input, patt ) )
-                    {
-                        MatchResult mr = matcher.getMatch();
-                        //System.out.println("***Match found for substitute***");
-                        // In this case we do a substitute
-                        result = Util.substitute(matcher, patt, substitution, str,
-                        Util.SUBSTITUTE_ALL);
-                        //System.out.println("New string: " + result);
-                        //outFile.println(result);
-                        str = result;
-                        subDone = true;
-                    }
-                }
-                if (subDone)
-                {
-                    //System.out.println("write the subbed line");
-                    outFile.println(result);
-                }
-                else
-                {
-                    //System.out.println("Write the str: " + str);
-                    outFile.println(str);
-                    outFile.flush();
+                    str = patt.matcher(str).replaceAll(sub);
                 }
+                //System.out.println("Write the str: " + str);
+                outFile.println(str);
             }// end if
         } // end while
         inFile.close();
