@@ -19,6 +19,7 @@
  */
 package org.apache.derbyTesting.junit;
 
+import org.apache.derbyTesting.functionTests.harness.JavaVersionHolder;
 import org.apache.derbyTesting.functionTests.util.PrivilegedFileOpsForTests;
 import junit.framework.Assert;
 import junit.framework.TestCase;
@@ -36,6 +37,7 @@ import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.io.InterruptedIOException;
 import java.io.RandomAccessFile;
+import java.lang.reflect.Field;
 import java.net.URL;
 import java.sql.SQLException;
 import java.security.AccessController;
@@ -135,6 +137,15 @@ public abstract class BaseTestCase
                 File origLog = new File(DEFAULT_DB_DIR, DERBY_LOG);
                 File newLog = new File(failPath, DERBY_LOG);
                 PrivilegedFileOpsForTests.copy(origLog, newLog);
+                // Copy some other likely files, the rolling log files
+                // These might occur if the tests are run with 
+                // derby.stream.error.style=rollingFile
+                for (int i=0; i < 10; i++) {
+                    String logName = "derby-" + i + ".log";
+                    File origRolLog = new File(DEFAULT_DB_DIR, logName);
+                    File newRolLog = new File(failPath, logName);
+                    PrivilegedFileOpsForTests.copy(origRolLog, newRolLog);
+                }
                 // Copy the database.
                 String dbName = TestConfiguration.getCurrent().getDefaultDatabaseName();
                 File dbDir = new File(DEFAULT_DB_DIR, dbName );
@@ -164,7 +175,7 @@ public abstract class BaseTestCase
                     System.exit(1);
                 }
                 else
-                throw running;
+                    throw running;
             }
         }
         finally{
@@ -242,7 +253,35 @@ public abstract class BaseTestCase
     }
 
     private final static PrintStream out = System.out;
-    
+
+    /**
+     * Change the value of {@code System.out}.
+     *
+     * @param out the new stream
+     */
+    protected static void setSystemOut(final PrintStream out) {
+        AccessController.doPrivileged(new PrivilegedAction<Void>() {
+            public Void run() {
+                System.setOut(out);
+                return null;
+            }
+        });
+    }
+
+    /**
+     * Change the value of {@code System.err}.
+     *
+     * @param err the new stream
+     */
+    protected static void setSystemErr(final PrintStream err) {
+        AccessController.doPrivileged(new PrivilegedAction<Void>() {
+            public Void run() {
+                System.setErr(err);
+                return null;
+            }
+        });
+    }
+
     /**
      * Set system property
      *
@@ -252,20 +291,14 @@ public abstract class BaseTestCase
     protected static void setSystemProperty(final String name, 
 					    final String value)
     {
-	
-	AccessController.doPrivileged
-	    (new java.security.PrivilegedAction(){
-		    
-		    public Object run(){
-			System.setProperty( name, value);
-			return null;
-			
-		    }
-		    
-		}
-	     );
-	
+        AccessController.doPrivileged(new PrivilegedAction<Void>() {
+            public Void run() {
+                System.setProperty(name, value);
+                return null;
+            }
+        });
     }
+
     /**
      * Remove system property
      *
@@ -273,20 +306,14 @@ public abstract class BaseTestCase
      */
     public static void removeSystemProperty(final String name)
 	{
-	
-	AccessController.doPrivileged
-	    (new java.security.PrivilegedAction(){
-		    
-		    public Object run(){
-			System.getProperties().remove(name);
-			return null;
-			
-		    }
-		    
-		}
-	     );
-	
-    }    
+        AccessController.doPrivileged(new PrivilegedAction<Void>() {
+            public Void run() {
+                System.getProperties().remove(name);
+                return null;
+            }
+        });
+    }
+
     /**
      * Get system property.
      *
@@ -294,17 +321,11 @@ public abstract class BaseTestCase
      */
     protected static String getSystemProperty(final String name)
 	{
-
-	return (String )AccessController.doPrivileged
-	    (new java.security.PrivilegedAction(){
-
-		    public Object run(){
-			return System.getProperty(name);
-
-		    }
-
-		}
-	     );
+        return AccessController.doPrivileged(new PrivilegedAction<String>() {
+            public String run() {
+                return System.getProperty(name);
+            }
+        });
     }
     
     /**
@@ -317,9 +338,8 @@ public abstract class BaseTestCase
      * @return The list indicates files with certain prefix.
      */
     protected static String[] getFilesWith(final File dir, String prefix) {
-        return (String[]) AccessController
-                .doPrivileged(new java.security.PrivilegedAction() {
-                    public Object run() {
+        return AccessController.doPrivileged(new PrivilegedAction<String[]>() {
+                    public String[] run() {
                         //create a FilenameFilter and override its accept-method to file
                         //files start with "javacore"*
                         FilenameFilter filefilter = new FilenameFilter() {
@@ -341,19 +361,12 @@ public abstract class BaseTestCase
      */
     protected static URL getTestResource(final String name)
 	{
-
-	return (URL)AccessController.doPrivileged
-	    (new java.security.PrivilegedAction(){
-
-		    public Object run(){
-			return BaseTestCase.class.getClassLoader().
-			    getResource(name);
-
-		    }
-
-		}
-	     );
-    }  
+        return AccessController.doPrivileged(new PrivilegedAction<URL>() {
+            public URL run() {
+                return BaseTestCase.class.getClassLoader().getResource(name);
+            }
+        });
+    }
   
     /**
      * Open the URL for a a test resource, e.g. a policy
@@ -364,16 +377,12 @@ public abstract class BaseTestCase
     protected static InputStream openTestResource(final URL url)
         throws PrivilegedActionException
     {
-    	return (InputStream)AccessController.doPrivileged
-	    (new java.security.PrivilegedExceptionAction(){
-
-		    public Object run() throws IOException{
-			return url.openStream();
-
-		    }
-
-		}
-	     );    	
+        return AccessController.doPrivileged(
+                new PrivilegedExceptionAction<InputStream>() {
+            public InputStream run() throws IOException {
+                return url.openStream();
+            }
+        });
     }
     
     /**
@@ -477,6 +486,40 @@ public abstract class BaseTestCase
                      t1.getMessage(), t2.getMessage());
     }
     
+    /**
+     * <p>
+     * Assert the equivalence of two byte arrays.
+     * </p>
+     */
+    public  static  void    assertEquals( byte[] expected, byte[] actual )
+    {
+        if ( assertSameNullness( expected, actual ) ) { return; }
+        
+        assertEquals( expected.length, actual.length );
+        for ( int i = 0; i < expected.length; i++ )
+        {
+            assertEquals( Integer.toString( i ), expected[ i ], actual[ i ] );
+        }
+    }
+
+    /**
+     * Assert that two objects are either both null or neither null.
+     * Returns true if they are null.
+     */
+    public  static  boolean assertSameNullness( Object expected, Object actual )
+    {
+        if ( expected ==  null )
+        {
+            assertNull( actual );
+            return true;
+        }
+        else
+        {
+            assertNotNull( actual );
+            return false;
+        }
+    }
+
     /**
      * <p>
      * Assert the equivalence of two int arrays.
@@ -484,6 +527,8 @@ public abstract class BaseTestCase
      */
     public  static  void    assertEquals( int[] expected, int[] actual )
     {
+        if ( assertSameNullness( expected, actual ) ) { return; }
+        
         assertEquals( expected.length, actual.length );
         for ( int i = 0; i < expected.length; i++ )
         {
@@ -498,6 +543,8 @@ public abstract class BaseTestCase
      */
     public  static  void    assertEquals( long[] expected, long[] actual )
     {
+        if ( assertSameNullness( expected, actual ) ) { return; }
+        
         assertEquals( expected.length, actual.length );
         for ( int i = 0; i < expected.length; i++ )
         {
@@ -513,8 +560,8 @@ public abstract class BaseTestCase
      */
 	public static void assertEquals(final File file1, final File file2) {
 		AccessController.doPrivileged
-        (new PrivilegedAction() {
-        	public Object run() {
+        (new PrivilegedAction<Void>() {
+        	public Void run() {
         		try {
 					InputStream f1 = new BufferedInputStream(new FileInputStream(file1));
 					InputStream f2 = new BufferedInputStream(new FileInputStream(file2));
@@ -566,6 +613,15 @@ public abstract class BaseTestCase
 	    }
 	}
 
+    /**
+     * Same as {@link #execJavaCmd( String, String, String[], File, boolean )}
+     * but with {@code addClassPath == true}.
+     */
+    public static Process execJavaCmd(
+        String jvm, String cp, String[] cmd, final File dir)
+            throws IOException {
+        return execJavaCmd(jvm, cp, cmd, dir, true);
+    }
 
 	/**
 	 * Execute a java command and return the process.
@@ -583,17 +639,18 @@ public abstract class BaseTestCase
 	 * @param cmd array of java arguments for command
      * @param dir working directory for the sub-process, or {@code null} to
      *            run in the same directory as the main test process
+     * @param addClassPath if {@code true},add classpath
 	 * @return the process that was started
 	 * @throws IOException
 	 */
     public static Process execJavaCmd(
-        String jvm, String cp, String[] cmd, final File dir)
+        String jvm, String cp, String[] cmd, final File dir, boolean addClassPath)
             throws IOException {
 
         // Is this an invocation of a jar file with java -jar ...?
         final boolean isJarInvocation = cmd.length > 0 && cmd[0].equals("-jar");
 
-	    ArrayList cmdlist = new ArrayList();
+	    ArrayList<String> cmdlist = new ArrayList<String>();
         cmdlist.add(jvm == null ? getJavaExecutableName() : jvm);
 	    if (isJ9Platform())
 	    {
@@ -646,6 +703,7 @@ public abstract class BaseTestCase
             String agent = getSystemProperty(JACOCO_AGENT_PROP);
             cmdlist.add(agent + (agent.endsWith("=") ? "": ",") +
                     "destfile=" + getJaCoCoOutFile());
+            cmdlist.add("-Djacoco.active=");
         }
 
         if (isSunJVM() && Boolean.valueOf(
@@ -660,7 +718,7 @@ public abstract class BaseTestCase
             // been set in addition to -jar, as that's probably a mistake in
             // the calling code.
             assertNull("Both -jar and classpath specified", cp);
-        } else {
+        } else if (addClassPath) {
             cmdlist.add("-classpath");
             cmdlist.add(cp == null ? getSystemProperty("java.class.path") : cp);
         }
@@ -674,13 +732,11 @@ public abstract class BaseTestCase
 	        println("command[" + i + "]" + command[i]);
 	    }
 	    try {
-	        return (Process) AccessController
-	        .doPrivileged(new PrivilegedExceptionAction() {
-	            public Object run() throws IOException {
-	                Process result = null;
-	                result = Runtime.getRuntime().exec(
+            return AccessController.doPrivileged(
+                    new PrivilegedExceptionAction<Process>() {
+                public Process run() throws IOException {
+                    return Runtime.getRuntime().exec(
                             command, (String[]) null, dir);
-	                return result;
 	            }
 	        });
 	    } catch (PrivilegedActionException pe) {
@@ -819,15 +875,11 @@ public abstract class BaseTestCase
     public static final boolean isJava5() {
         return getSystemProperty("java.version").startsWith("1.5");
     }
-   
+
     public static final boolean isJava7() {
         return getSystemProperty("java.version").startsWith("1.7");
     }
 
-    public static final boolean isJava8() {
-        return getSystemProperty("java.version").startsWith("1.8");
-    }
-
     public static final boolean runsWithEmma() {
         return getSystemProperty("java.class.path").indexOf("emma.jar") != -1;
     }
@@ -909,7 +961,7 @@ public abstract class BaseTestCase
 
     /**
      * Check if we have old style (before Sun Java 1.7) Solaris interruptible
-     * IO. On Sun Java 1.5 >= update 22 and Sun Java 1.6 this can be disabled
+     * IO. On Sun Java 1.5 &gt;= update 22 and Sun Java 1.6 this can be disabled
      * with Java option {@code -XX:-UseVMInterruptibleIO}. On Sun Java 1.7 it
      * is by default disabled.
      *
@@ -921,8 +973,8 @@ public abstract class BaseTestCase
 
         try {
             AccessController.doPrivileged(
-                new PrivilegedExceptionAction() {
-                    public Object run() throws
+                new PrivilegedExceptionAction<Void>() {
+                    public Void run() throws
                         IOException, InterruptedIOException {
 
                         TestConfiguration curr = TestConfiguration.getCurrent();
@@ -1110,12 +1162,23 @@ public abstract class BaseTestCase
      */
     public static void fail(String msg, Throwable t)
             throws AssertionFailedError {
-
-        AssertionFailedError ae = new AssertionFailedError(msg);
-        ae.initCause(t);
-        throw ae;
+        throw newAssertionFailedError(msg, t);
     }
-    
+
+    /**
+     * Create a new AssertionFailedError linked to another Throwable.
+     *
+     * @param message message explaining the failure
+     * @param cause the cause of the failure
+     * @return an AssertionFailedError
+     */
+    public static AssertionFailedError newAssertionFailedError(
+            String message, Throwable cause) {
+        AssertionFailedError e = new AssertionFailedError(message);
+        e.initCause(cause);
+        return e;
+    }
+
     /**
      * assert a method from an executing test
      * 
@@ -1158,6 +1221,22 @@ public abstract class BaseTestCase
         }
     }
 
+    /** Return true if the JVM is at least at the indicated rev level */
+    public static boolean vmAtLeast( int major, int minor )
+    {
+        String version = AccessController.doPrivileged
+            (new PrivilegedAction<String>(){
+                public String run(){
+                    return System.getProperty( "java.version" );
+                }
+            }
+                );
+                   
+        JavaVersionHolder jvh =  new JavaVersionHolder( version );
+
+        return jvh.atLeast( major, minor );
+    }
+
     /** Makes the current thread sleep up to {@code ms} milliseconds. */
     public static void sleep(long ms) {
         try {
@@ -1176,7 +1255,7 @@ public abstract class BaseTestCase
         }
     }
     
-    private static void setupForDebuggerAttach(ArrayList cmdlist) {
+    private static void setupForDebuggerAttach(ArrayList<String> cmdlist) {
         if (debugPort == 0) {
             // lazy initialization
             String dbp = getSystemProperty("derby.test.debugPortBase");
